SystemOrganization addCategory: #'Protocols-Kernel'!
SystemOrganization addCategory: #'Protocols-Tools'!
SystemOrganization addCategory: #'Protocols-Type Vocabularies'!

!ClassDescription methodsFor: '*Protocols-Tools' stamp: 'nice 10/20/2009 20:48'!
chooseClassVarName 
	"Present the user with a list of class variable names and answer the one selected, or nil if none"

	| lines labelStream allVars index |
	lines := OrderedCollection new.
	allVars := OrderedCollection new.
	labelStream := WriteStream on: (String new: 200).
	self withAllSuperclasses reverseDo:
		[:class | | vars |
		vars := class classVarNames.
		vars do:
			[:var |
			labelStream nextPutAll: var; cr.
			allVars add: var].
		vars isEmpty ifFalse: [lines add: allVars size]].
	labelStream contents isEmpty ifTrue: [^Beeper beep]. "handle nil superclass better"
	labelStream skip: -1 "cut last CR".
	index := (UIManager default chooseFrom: (labelStream contents substrings) lines: lines).
	index = 0 ifTrue: [^ nil].
	^ allVars at: index! !

!ObjectExplorer methodsFor: '*Protocols-Tools' stamp: 'mt 2/3/2016 20:13'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection.  If in mvc, an old-style protocol browser is opened instead."

	Smalltalk isMorphic
		ifFalse: [ProtocolBrowser openSubProtocolForClass: self selectedClass]
		ifTrue: [InstanceBrowser new openOnObject: self object showingSelector: nil].! !

!Inspector methodsFor: '*Protocols-Tools' stamp: 'ul 11/15/2010 11:43'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection.  If in mvc, an old-style protocol browser is opened instead."

	| objectToRepresent |
	Smalltalk isMorphic ifFalse: [^ self spawnProtocol].

	objectToRepresent := self selectionIndex = 0 ifTrue: [object] ifFalse: [self selection].
	InstanceBrowser new openOnObject: objectToRepresent showingSelector: nil! !

!Inspector methodsFor: '*Protocols-Tools' stamp: 'ul 11/15/2010 11:44'!
spawnFullProtocol
	"Spawn a window showing full protocol for the receiver's selection"

	| objectToRepresent |
	objectToRepresent := self selectionIndex = 0 ifTrue: [object] ifFalse: [self selection].
	ProtocolBrowser openFullProtocolForClass: objectToRepresent class! !

!Inspector methodsFor: '*Protocols-Tools' stamp: 'ul 11/15/2010 11:44'!
spawnProtocol
	"Spawn a protocol on browser on the receiver's selection"

	| objectToRepresent |
	objectToRepresent := self selectionIndex = 0 ifTrue: [object] ifFalse: [self selection].
	ProtocolBrowser openSubProtocolForClass: objectToRepresent class! !

!PasteUpMorph methodsFor: '*Protocols' stamp: 'sw 7/10/2005 03:55'!
abandonVocabularyPreference
	"Remove any memory of a preferred vocabulary in the project"

	| standardViewers aVocabulary |
	self removeProperty: #currentVocabularySymbol.

	standardViewers := (self submorphsSatisfying: [:m | m isKindOf: ViewerFlapTab]) collect:
		[:m | m referent firstSubmorph].
	aVocabulary := Vocabulary vocabularyNamed: #eToy.
	standardViewers do:
		[:m | ((m valueOfProperty: #currentVocabularySymbol ifAbsent: [nil]) == #Vector) ifTrue:
			[m switchToVocabulary: aVocabulary]]

"ActiveWorld abandonVocabularyPreference"! !

!PasteUpMorph methodsFor: '*Protocols' stamp: 'sw 6/4/2001 19:38'!
currentVocabulary
	"Answer the default Vocabulary object to be applied when scripting"

	| aSym aVocab |
	aSym := self valueOfProperty: #currentVocabularySymbol.
	aSym ifNil:
		[aVocab := self valueOfProperty: #currentVocabulary.
		aVocab ifNotNil:
			[aSym := aVocab vocabularyName.
			self setProperty: #currentVocabularySymbol toValue: aSym.
			self removeProperty: #currentVocabulary]].
	^ aSym
		ifNotNil:
			[Vocabulary vocabularyNamed: aSym]
		ifNil:
			[Vocabulary fullVocabulary]! !

!PasteUpMorph methodsFor: '*Protocols' stamp: 'sw 7/9/2005 12:58'!
installVectorVocabulary
	"Install the experimental Vector vocabulary as the default for the current project"

	| standardViewers aVocabulary |
	self setProperty: #currentVocabularySymbol toValue: #Vector.
	standardViewers := (self submorphsSatisfying: [:m | m isKindOf: ViewerFlapTab]) collect:
		[:m | m referent firstSubmorph].
	aVocabulary := Vocabulary vocabularyNamed: #Vector.
	standardViewers do: [:m | m switchToVocabulary: aVocabulary]! !

!PasteUpMorph methodsFor: '*Protocols' stamp: 'sw 7/10/2005 02:53'!
makeVectorUseConformToPreference
	"Make certain that the use of vectors in this project conforms to the current preference setting."

	| prefValue currentValue |
	prefValue := Preferences useVectorVocabulary.
	currentValue := self currentlyUsingVectorVocabulary.
	prefValue ~~ currentValue ifTrue:
		[currentValue
			ifTrue:
				[self abandonVocabularyPreference]
			ifFalse:
				[self installVectorVocabulary]]! !

!CodeHolder methodsFor: '*Protocols-Tools' stamp: 'sw 12/4/2000 12:07'!
spawnFullProtocol
	"Create and schedule a new protocol browser on the currently selected class or meta."

	| aClassOrMetaclass |
	(aClassOrMetaclass := self selectedClassOrMetaClass) ifNotNil:
       	[ProtocolBrowser openFullProtocolForClass: aClassOrMetaclass]! !

!CodeHolder methodsFor: '*Protocols-Tools' stamp: 'sw 5/8/2000 14:24'!
spawnProtocol
	| aClassOrMetaclass |
	"Create and schedule a new protocol browser on the currently selected class or meta."
	(aClassOrMetaclass := self selectedClassOrMetaClass) ifNotNil:
       	[ProtocolBrowser openSubProtocolForClass: aClassOrMetaclass]! !

!StringHolder methodsFor: '*Protocols-Tools' stamp: 'ar 3/5/2010 21:01'!
browseFullProtocol
	"Open up a protocol-category browser on the value of the receiver's current selection.    If in mvc, an old-style protocol browser is opened instead.  Someone who still uses mvc might wish to make the protocol-category-browser work there too, thanks."

	| aClass |

	(Smalltalk isMorphic and: [Smalltalk hasClassNamed: #Lexicon]) ifFalse: [^ self spawnFullProtocol].
	((aClass := self selectedClassOrMetaClass) notNil and: [aClass isTrait not]) ifTrue:
		[(Smalltalk at: #Lexicon) new openOnClass: aClass showingSelector: self selectedMessageName]! !

!Behavior methodsFor: '*Protocols' stamp: 'sw 5/4/2001 07:44'!
implementsVocabulary: aVocabulary
	"Answer whether instances of the receiver respond to the messages in aVocabulary."

	(aVocabulary isKindOf: FullVocabulary orOf: ScreenedVocabulary) ifTrue: [^ true].
	^ self fullyImplementsVocabulary: aVocabulary! !

MessageSend subclass: #MethodCall
	instanceVariableNames: 'lastValue methodInterface timeStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!MethodCall commentStamp: '<historical>' prior: 0!
A MethodCall is a resendable message-send, complete with receiver, instantiated arguments, and a memory of when it was last evaluated and what the last value was.  

The methodInterface with which it is associated can furnish argument names, documentation, and other information.!

!MethodCall methodsFor: 'method interface' stamp: ''!
ephemeralMethodInterface
	"Answer a methodInterface for me. If I have one stored, answer it; if 
	not, conjure up an interface and answer it but do NOT store it 
	internally. You can call this directly if you need a method interface 
	for me but do not want any conjured-up interface to persist."
	^ methodInterface
		ifNil: [MethodInterface new
				conjuredUpFor: selector
				class: (self receiver class whichClassIncludesSelector: selector)]! !

!MethodCall methodsFor: 'evaluation' stamp: 'sw 11/20/2001 12:15'!
evaluate
	"Evaluate the receiver, and if value has changed, signal value-changed"

	| result |
	result := arguments isEmptyOrNil
		ifTrue: [self receiver perform: selector]
		ifFalse: [self receiver perform: selector withArguments: arguments asArray].
	timeStamp := Time dateAndTimeNow.
	result ~= lastValue ifTrue:
		[lastValue := result.
		self changed: #value]
	! !

!MethodCall methodsFor: 'evaluation' stamp: 'sw 11/20/2001 12:21'!
everEvaluated
	"Answer whether this method call has ever been evaluated"

	^ timeStamp notNil! !

!MethodCall methodsFor: 'evaluation' stamp: 'sw 11/20/2001 13:31'!
lastValue
	"Answer the last value I remember obtaining from an evaluation"

	^ lastValue! !

!MethodCall methodsFor: 'method interface' stamp: 'sw 11/20/2001 12:43'!
methodInterface
	"Answer the receiver's methodInterface, conjuring one up on the spot (and remembering) if not present"

	^ methodInterface ifNil:
		[methodInterface := self ephemeralMethodInterface]! !

!MethodCall methodsFor: 'method interface' stamp: 'sw 11/20/2001 12:40'!
methodInterfaceOrNil
	"Answer my methodInterface, whether it is nil or not"

	^ methodInterface! !

!MethodCall methodsFor: 'method interface' stamp: 'sw 11/20/2001 12:39'!
methodInterface: anInterface
	"Set my methodInterface"

	methodInterface := anInterface! !

!MethodCall methodsFor: 'initialization' stamp: 'sw 11/20/2001 13:34'!
receiver: aReceiver methodInterface: aMethodInterface
	"Initialize me to have the given receiver and methodInterface"

	| aResultType |
	receiver := aReceiver.
	selector := aMethodInterface selector.
	methodInterface := aMethodInterface.
	arguments := aMethodInterface defaultArguments.

	self flag: #noteToTed.
	"the below can't really survive, I know.  The intent is that if the method has a declared result type, we want the preferred readout type to be able to handle the initial #lastValue even if the MethodCall has not been evaluated yet; thus we'd rather have a boolean value such as true rather than a nil here if we're showing a boolean readout such as a checkbox, and likewise for color-valued and numeric-valued readouts etc, "

	(aResultType := methodInterface resultType) ~~ #unknown ifTrue:
		[lastValue := (Vocabulary vocabularyForType: aResultType) initialValueForASlotFor: aReceiver]        ! !

!MethodCall methodsFor: 'initialization' stamp: 'sw 10/3/2001 15:28'!
receiver: aReceiver methodInterface: aMethodInterface initialArguments: initialArguments
	"Set up a method-call for the given receiver, method-interface, and initial arguments"

	receiver := aReceiver.
	selector := aMethodInterface selector.
	methodInterface := aMethodInterface.
	arguments := initialArguments ifNotNil: [initialArguments asArray]
! !

!MethodCall methodsFor: 'argument access' stamp: 'sw 11/20/2001 12:16'!
setArgumentNamed: aName toValue: aValue
	"Set the argument of the given name to the given value"

	| anIndex |
	anIndex := self methodInterface argumentVariables findFirst:
		[:aVariable | aVariable variableName = aName].
	anIndex > 0
		ifTrue:
			[arguments at: anIndex put: aValue]
		ifFalse:
			[self error: 'argument missing'].
	self changed: #argumentValue! !

!MethodCall methodsFor: 'initialization' stamp: 'sw 11/20/2001 12:16'!
valueOfArgumentNamed: aName
	"Answer the value of the given arguement variable"

	| anIndex |
	anIndex := self methodInterface argumentVariables findFirst:
		[:aVariable | aVariable variableName = aName].
	^ anIndex > 0
		ifTrue:
			[arguments at: anIndex]
		ifFalse:
			[self error: 'variable not found']! !

MessageSet subclass: #ProtocolBrowser
	instanceVariableNames: 'selectedClass selectedSelector'
	classVariableNames: 'TextMenu'
	poolDictionaries: ''
	category: 'Protocols-Tools'!

!ProtocolBrowser commentStamp: '<historical>' prior: 0!
An instance of ProtocolBrowser shows the methods a class understands--inherited or implemented at this level--as a "flattened" list.!

ProtocolBrowser subclass: #Lexicon
	instanceVariableNames: 'currentVocabulary categoryList categoryListIndex targetClass limitClass currentQuery currentQueryParameter selectorsVisited'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Tools'!

!Lexicon commentStamp: 'fbs 5/2/2013 08:29' prior: 0!
An instance of Lexicon shows the list of all the method categories known to an object or any of its superclasses, as a "flattened" list, and, within any selected category, shows all methods understood by the class's instances which are associated with that category, again as a "flattened" list.  A variant with a search pane rather than a category list is also implemented.

categoryList				the list of categories
categoryListIndex		index of currently-selected category
targetObject				optional -- an instance being viewed
targetClass				the class being viewed
lastSearchString			the last string searched for
lastSendersSearchSelector	the last senders search selector
limitClass				optional -- the limit class to search for
selectorsVisited			list of selectors visited
selectorsActive			not presently in use, subsumed by selectorsVisited
currentVocabulary		the vocabulary currently installed
currentQuery			what the query category relates to:
							#senders #selectorName #currentChangeSet!

Lexicon subclass: #InstanceBrowser
	instanceVariableNames: 'objectViewed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Tools'!

!InstanceBrowser methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.726 g: 0.9 b: 0.9)! !

!InstanceBrowser methodsFor: 'menu commands' stamp: 'sw 3/20/2001 13:20'!
inspectViewee
	"Open an Inspector on the object I view"

	objectViewed inspect! !

!InstanceBrowser methodsFor: 'menu commands' stamp: 'cmm 10/16/2015 14:15'!
offerMenu
	"Offer a menu to the user, in response to the hitting of the menu button on the tool pane"

	| aMenu |
	aMenu := MenuMorph new defaultTarget: self.
	aMenu title: 'Messages of ', objectViewed nameForViewer.
	aMenu addStayUpItem.
	aMenu addList: #(
		('vocabulary...' 			chooseVocabulary)
		('what to show...'			offerWhatToShowMenu)
		-
		('inst var refs (here)'		setLocalInstVarRefs)
		('inst var defs (here)'		setLocalInstVarDefs)
		('class var refs (here)'		setLocalClassVarRefs)
		-

		('navigate to a sender...' 	navigateToASender)
		('recent...' 					navigateToRecentMethod)
		('show methods in current change set'
									showMethodsInCurrentChangeSet)
		('show methods with initials...'
									showMethodsWithInitials)
		-
		"('toggle search pane' 		toggleSearch)"

		-
		-
		('browse full (b)' 			browseMethodFull)
		('browse hierarchy (h)'		classHierarchy)
		('browse protocol (p)'		browseFullProtocol)
		-
		('fileOut'					fileOutMessage)
		('printOut'					printOutMessage)
		-
		('senders of... (n)'			browseSendersOfMessages)
		('implementors of... (m)'		browseMessages)
		('versions (v)' 				browseVersions)
		('inheritance (i)'			methodHierarchy)
		-
		('references... (r)' 				browseVariableReferences)
		('assignments... (a)' 				browseVariableAssignments)
		-
		('viewer on me'				viewViewee)
		('inspector on me'			inspectViewee)
		-
		('more...'					shiftedYellowButtonActivity)).

	aMenu popUpInWorld: ActiveWorld! !

!InstanceBrowser methodsFor: 'initialization' stamp: 'ar 8/8/2009 00:31'!
openOnObject: anObject inWorld: ignored showingSelector: aSelector
	"Create and open a SystemWindow to house the receiver, showing the categories pane."
	^self openOnObject: anObject showingSelector: aSelector! !

!InstanceBrowser methodsFor: 'initialization' stamp: 'ar 8/7/2009 23:26'!
openOnObject: anObject showingSelector: aSelector
	"Create and open a SystemWindow to house the receiver, showing the categories pane."

	objectViewed := anObject.
	self openOnClass: anObject class showingSelector: aSelector! !

!InstanceBrowser methodsFor: 'window title' stamp: 'sw 3/20/2001 12:18'!
startingWindowTitle
	"Answer the initial window title to apply"

	^ 'Vocabulary of ', objectViewed nameForViewer! !

!InstanceBrowser methodsFor: 'target-object access' stamp: 'sw 3/20/2001 12:10'!
targetObject
	"Answer the object to which this tool is bound"

	^ objectViewed! !

!InstanceBrowser methodsFor: 'menu commands' stamp: 'sw 3/20/2001 13:19'!
viewViewee
	"Open a viewer on the object I view"

	objectViewed beViewed! !

!Lexicon class methodsFor: 'visible category names' stamp: 'sw 12/14/2000 14:15'!
activeCategoryName
	"Answer the name to be used for the active-methods category"

	true ifTrue: [^ #'-- current working set --'].

	'-- current working set --' asSymbol "Placed here so a message-strings-containing-it query will find this method"
! !

!Lexicon class methodsFor: 'visible category names' stamp: 'sw 12/13/2000 10:56'!
allCategoryName
	"Answer the name to be used for the all category"

	true ifTrue: [^ #'-- all --'].

	'-- all --' asSymbol  "Placed here so a message-strings-containing-it query will find this method"
! !

!Lexicon class methodsFor: 'visible category names' stamp: 'sw 3/19/2001 08:17'!
queryCategoryName
	"Answer the name to be used for the query-results category"

	true ifTrue: [^ #'-- query results --'].

	^ '-- query results --' asSymbol   "Placed here so a message-strings-containing-it query will find this method"! !

!Lexicon class methodsFor: 'visible category names' stamp: 'sw 12/13/2000 10:54'!
sendersCategoryName
	"Answer the name to be used for the senders-results category"

	true ifTrue: [^ #'-- "senders" results --'].

	^ '-- "senders" results --'.  "so methods-strings-containing will find this"! !

!Lexicon class methodsFor: 'visible category names' stamp: 'sw 3/19/2001 08:03'!
viewedCategoryName
	"Answer the name to be used for the previously-viewed-methods category"

	true ifTrue: [^ #'-- active --'].

	^ '-- active --' asSymbol	 "For benefit of method-strings-containing-it search"
! !

!Lexicon methodsFor: 'window title' stamp: 'sw 3/19/2001 08:45'!
addModelItemsToWindowMenu: aMenu
	"Add model-related item to the window menu"

	super addModelItemsToWindowMenu: aMenu. 
	aMenu add: 'choose vocabulary...' target: self action: #chooseVocabulary! !

!Lexicon methodsFor: 'toolbuilder' stamp: 'rkrk 8/23/2009 05:31'!
addSpecialButtonsTo: buttonPanelSpec with: builder

	| homeCatBtnSpec menuBtnSpec mostGenericBtnSpec |
	homeCatBtnSpec := builder pluggableButtonSpec new
		model: self;
		action: #showHomeCategory;
		label: (ScriptingSystem formAtKey: #Cat) asMorph;
		help: 'show this method''s home category';
		yourself.
	menuBtnSpec := builder pluggableButtonSpec new
		model: self;
		action: #offerMenu;
		label: (ScriptingSystem formAtKey: #TinyMenu) asMorph;
		help: 'click here to get a menu with further options';
		yourself.
	mostGenericBtnSpec :=builder pluggableButtonSpec new
		model: self;
		action: #chooseLimitClass;
		label: #limitClassString;
		help: 'Governs which classes'' methods should be shown.  If this is the same as the viewed class, then only methods implemented in that class will be shown.  If it is ProtoObject, then methods of all classes in the vocabulary will be shown.'.
	buttonPanelSpec children
		add: homeCatBtnSpec;
		addFirst: mostGenericBtnSpec;
		addFirst: menuBtnSpec.! !

!Lexicon methodsFor: 'window title' stamp: 'sw 3/20/2001 16:42'!
adjustWindowTitle
	"Set the title of the receiver's window, if any, to reflect the current choices"

	| aWindow aLabel catName |
	(catName := self selectedCategoryName) ifNil: [^ self].
	(aWindow := self containingWindow) ifNil: [^ self].
	aLabel := nil.
	#(	(viewedCategoryName		'Messages already viewed - ')
		(allCategoryName			'All messages - ')) do:
			[:aPair | catName = (self categoryWithNameSpecifiedBy: aPair first) ifTrue: [aLabel := aPair second]].

	aLabel ifNil:
		[aLabel := catName = self class queryCategoryName
			ifTrue:
				[self queryCharacterization, ' - ']
			ifFalse:
				['Vocabulary of ']].
	aWindow setLabel: aLabel, (self targetObject ifNil: [targetClass]) nameForViewer! !

!Lexicon methodsFor: 'basic operation' stamp: 'sw 3/20/2001 16:06'!
annotation
	"Provide a line of annotation material for a middle pane."

	| aCategoryName |
	self selectedMessageName ifNotNil: [^ super annotation].
	(aCategoryName := self selectedCategoryName) ifNil:
		[^ self hasSearchPane
			ifTrue:
				['type a message name or fragment in the top pane and hit RETURN or ENTER']
			ifFalse:
				[''  "currentVocabulary documentation"]].


	(aCategoryName = self class queryCategoryName) ifTrue:
		[^ self queryCharacterization].
		
	#(
	(allCategoryName			'Shows all methods, whatever other category they belong to')
	(viewedCategoryName		'Methods visited recently.  Use  "-" button to remove a method from this category.')
	(queryCategoryName		'Query results'))

		do:
			[:pair | (self categoryWithNameSpecifiedBy: pair first) = aCategoryName ifTrue: [^ pair second]].

	^ currentVocabulary categoryCommentFor: aCategoryName! !

!Lexicon methodsFor: 'new-window queries' stamp: 'cmm 5/2/2011 09:55'!
browseVariableAssignments
	"Let the search pertain to the target class regardless of selection."
	self systemNavigation browseVariableAssignments: targetClass! !

!Lexicon methodsFor: 'new-window queries' stamp: 'cmm 5/2/2011 09:45'!
browseVariableReferences
	"Let the search pertain to the target class regardless of selection"
	self systemNavigation  browseVariableReferences: targetClass! !

!Lexicon methodsFor: 'toolbuilder' stamp: 'rkrk 8/23/2009 03:04'!
buildCategoryListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #categoryList; 
		getIndex: #categoryListIndex; 
		setIndex: #categoryListIndex:; 
		menu: #categoryListMenu:shifted:; 
		keyPress: #categoryListKey:from:.
	^listSpec! !

!Lexicon methodsFor: 'toolbuilder' stamp: 'nice 12/27/2009 03:10'!
buildCustomButtonsWith: builder

	"This method if very similar to StringHolder>>buildOptionalButtonsWith:.
	Refactor and pass in button specs?"
	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	self customButtonSpecs do: [:spec | | buttonSpec |
		buttonSpec := builder pluggableActionButtonSpec new.
		buttonSpec model: self.
		buttonSpec label: spec first.
		buttonSpec action: spec second.
		spec size > 2 ifTrue: [buttonSpec help: spec third].
		panelSpec children add: buttonSpec.
	].
	panelSpec layout: #horizontal. "buttons"
	self addSpecialButtonsTo: panelSpec with: builder.
	^panelSpec! !

!Lexicon methodsFor: 'toolbuilder' stamp: 'fbs 4/14/2010 12:34'!
buildWith: builder
	"Create the ui for the browser"
	| windowSpec max |
	max := self wantsOptionalButtons ifTrue:[0.32] ifFalse:[0.4].
	windowSpec := self buildWindowWith: builder specs: {
		(self listFrame: max fromLeft: 0 width: 0.5) -> [self buildCategoryListWith: builder].
		(self listFrame: max fromLeft: 0.5 width: 0.5) -> [self buildMessageListWith: builder].
		(self customButtonsFrame: max) -> [self buildCustomButtonsWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}.
	^builder build: windowSpec! !

!Lexicon methodsFor: 'message category functions' stamp: 'sw 10/8/2001 14:25'!
canShowMultipleMessageCategories
	"Answer whether the receiver is capable of showing multiple message categories"

	^ true! !

!Lexicon methodsFor: 'category list' stamp: 'sw 3/7/2001 12:19'!
categoriesPane
	"If there is a pane defined by #categoryList in my containing window, answer it, else answer nil"

	^ self listPaneWithSelector: #categoryList! !

!Lexicon methodsFor: 'category list' stamp: 'sw 3/20/2001 12:13'!
categoryDefiningSelector: aSelector
	"Answer a category in which aSelector occurs"

	| categoryNames |
	categoryNames := categoryList copyWithoutAll: #('-- all --').
	^ currentVocabulary categoryWithNameIn: categoryNames thatIncludesSelector: aSelector forInstance: self targetObject ofClass: targetClass! !

!Lexicon methodsFor: 'category list' stamp: 'sw 5/25/2001 01:34'!
categoryList
	"Answer the category list for the protcol, creating it if necessary, and prepending the -- all -- category, and appending the other special categories for search results, etc."

	| specialCategoryNames |
	categoryList ifNil:
		[specialCategoryNames := #(queryCategoryName  viewedCategoryName "searchCategoryName sendersCategoryName  changedCategoryName activeCategoryName")  collect:
			[:sym | self class perform: sym].
		categoryList :=
			(currentVocabulary categoryListForInstance: self targetObject ofClass: targetClass limitClass: limitClass),
			specialCategoryNames,
			(Array with: self class allCategoryName)].
	^ categoryList! !

!Lexicon methodsFor: 'category list' stamp: 'sw 12/12/2000 19:37'!
categoryListIndex
	"Answer the index of the currently-selected item in in the category list"

	^ categoryListIndex ifNil: [categoryListIndex := 1]! !

!Lexicon methodsFor: 'category list' stamp: 'sw 3/20/2001 20:19'!
categoryListIndex: anIndex
	"Set the category list index as indicated"

	| categoryName aList found existingSelector |
	existingSelector := self selectedMessageName.

	categoryListIndex := anIndex.
	anIndex > 0
		ifTrue:
			[categoryName := categoryList at: anIndex]
		ifFalse:
			[contents := nil].
	self changed: #categoryListIndex.

	found := false.
	#(	(viewedCategoryName		selectorsVisited)
		(queryCategoryName		selectorsRetrieved)) do:
			[:pair |
				categoryName = (self class perform: pair first)
					ifTrue:
						[aList := self perform: pair second.
						found := true]].
	found ifFalse:
		[aList := currentVocabulary allMethodsInCategory: categoryName forInstance: self targetObject ofClass: targetClass].
	categoryName = self class queryCategoryName ifFalse: [autoSelectString := nil].

	self initListFrom: aList highlighting: targetClass.

	messageListIndex := 0.
	self changed: #messageList.
	contents := nil.
	self contentsChanged.
	self selectWithinCurrentCategoryIfPossible: existingSelector.
	self adjustWindowTitle! !

!Lexicon methodsFor: 'category list' stamp: 'sw 12/12/2000 11:50'!
categoryListKey: aChar from: aView
	"The user hit a command-key while in the category-list.  Do something"

	(aChar == $f and: [self hasSearchPane not]) ifTrue:
		[^ self obtainNewSearchString].! !

!Lexicon methodsFor: 'category list' stamp: 'sw 12/1/2000 22:13'!
categoryListMenuTitle
	"Answer the menu title for the category list menu"

	^ 'categories'! !

!Lexicon methodsFor: 'category list' stamp: 'topa 3/18/2015 10:51'!
categoryListMenu: aMenu shifted: aBoolean
	"Answer the menu for the category list"
	^ self menu: aMenu for: #(categoryListMenu categoryListMenuShifted:) shifted: aBoolean
! !

!Lexicon methodsFor: 'selection' stamp: 'sd 11/20/2005 21:27'!
categoryOfSelector: aSelector 
	"Answer the name of the defining category for aSelector, or nil if none"
	| classDefiningSelector |
	classDefiningSelector := targetClass whichClassIncludesSelector: aSelector.
	classDefiningSelector
		ifNil: [^ nil].
	"can happen for example if one issues this from a change-sorter for a 
	message that is recorded as having been removed"
	^ classDefiningSelector whichCategoryIncludesSelector: aSelector! !

!Lexicon methodsFor: 'category list' stamp: 'sw 12/13/2000 10:38'!
categoryWithNameSpecifiedBy: aSelector
	"Answer the category name obtained by sending aSelector to my class.  This provides a way to avoid hard-coding the wording of conventions such as '-- all --'"

	^ self class perform: aSelector! !

!Lexicon methodsFor: 'category list' stamp: 'nb 6/17/2003 12:25'!
chooseCategory: aCategory
	"Choose the category of the given name, if there is one"

	self categoryListIndex: (categoryList indexOf: aCategory ifAbsent: [^ Beeper beep])! !

!Lexicon methodsFor: 'limit class' stamp: 'sw 3/19/2001 06:41'!
chooseLimitClass
	"Put up a menu allowing the user to choose the most generic class to show"

	| aMenu |
	aMenu := MenuMorph new defaultTarget: self.
	targetClass withAllSuperclasses do:
		[:aClass | 
			aClass == ProtoObject
				ifTrue:
					[aMenu addLine].
			aMenu add: aClass name selector: #setLimitClass: argument: aClass.
			aClass == limitClass ifTrue:
				[aMenu lastItem color: Color red].
			aClass == targetClass ifTrue: [aMenu addLine]].
	aMenu addTitle: 'Show only methods
implemented at or above...'.  "heh heh -- somebody please find nice wording here!!"
	aMenu popUpInWorld: self currentWorld! !

!Lexicon methodsFor: 'vocabulary' stamp: 'nice 2/4/2010 21:10'!
chooseVocabulary
	"Put up a dialog affording the user a chance to choose a different vocabulary to be installed in the receiver"
	
	Smalltalk at: #Vocabulary ifPresent:[:aClass|
		| aMenu |
		aMenu := MenuMorph new defaultTarget: self.
		aMenu addTitle: 'Choose a vocabulary
blue = current
red = imperfect' translated.
		aMenu addStayUpItem.
		aClass allStandardVocabularies do:[:aVocabulary |
			(targetClass implementsVocabulary: aVocabulary)
				ifTrue:
					[aMenu add: aVocabulary vocabularyName selector: #switchToVocabulary: argument: aVocabulary.
					(targetClass fullyImplementsVocabulary: aVocabulary) ifFalse:
						[aMenu lastItem color: Color red].
					aVocabulary == currentVocabulary ifTrue:
						[aMenu lastItem color: Color blue]. 
					aMenu balloonTextForLastItem: aVocabulary documentation]].
		aMenu popUpInWorld: self currentWorld
	].! !

!Lexicon methodsFor: 'contents' stamp: 'tk 9/14/2001 16:37'!
contents
	"We have a class, allow new messages to be defined"

	editSelection == #newMessage ifTrue: [^ targetClass sourceCodeTemplate].
	^ super contents! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 3/20/2001 18:59'!
currentQueryParameter
	"Answer the current query parameter"

	^ currentQueryParameter ifNil: [currentQueryParameter := 'contents']! !

!Lexicon methodsFor: 'control buttons' stamp: 'sw 7/23/2002 12:51'!
customButtonSpecs
	"Answer a triplet defining buttons, in the format:

			button label
			selector to send
			help message"
	| aa |
	aa := contentsSymbol == #tiles ifTrue: [{   "Consult Ted Kaehler regarding this bit"
	{'tiles'.				#tilesMenu.					'tiles for assignment and constants'. 	true}.
	{'vars'.				#varTilesMenu.	'tiles for instance variables and a new temporary'. 	true}
		}] ifFalse: [#()].	"true in 4th place means act on mouseDown"

	^ aa, #(
	('follow'			seeAlso							'view a method I implement that is called by this method')
	('find'				obtainNewSearchString			'find methods by name search')
	('sent...'			setSendersSearch				'view the methods I implement that send a given message')

	('<'					navigateToPreviousMethod 		'view the previous active method')
	('>'					navigateToNextMethod 			'view the next active method')
	('-'					removeFromSelectorsVisited		'remove this method from my active list'))! !

!Lexicon methodsFor: 'initialization' stamp: 'fbs 4/14/2010 11:40'!
customButtonsFrame: bottomFraction
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: bottomFraction offset: self buttonHeight negated - 4;
		rightFraction: 1 offset: 0;
		bottomFraction: bottomFraction offset: 0! !

!Lexicon methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.79 g: 0.9 b: 0.79)! !

!Lexicon methodsFor: 'basic operation' stamp: 'nb 6/17/2003 12:25'!
displaySelector: aSelector
	"Set aSelector to be the one whose source shows in the browser.  If there is a category list, make it highlight a suitable category"

	| detectedItem messageIndex |
	self chooseCategory: (self categoryDefiningSelector: aSelector).
	detectedItem := messageList detect:
		[:anItem | (anItem asString upTo: $ ) asSymbol == aSelector] ifNone: [^ Beeper beep].
	messageIndex := messageList indexOf: detectedItem.
	self messageListIndex: messageIndex! !

!Lexicon methodsFor: 'model glue' stamp: 'nice 3/31/2011 21:51'!
doItReceiver
	"This class's classPool has been jimmied to be the classPool of the class being browsed. A doIt in the code pane will let the user see the value of the class variables.  Here, if the receiver is affiliated with a specific instance, we give give that primacy"

	^ self targetObject ifNil: [self selectedClass]! !

!Lexicon methodsFor: 'search' stamp: 'sw 12/11/2000 15:26'!
hasSearchPane
	"Answer whether receiver has a search pane"

	^ self searchPane notNil! !

!Lexicon methodsFor: 'limit class' stamp: 'cmm 3/26/2003 22:33'!
initialLimitClass
	"Choose a plausible initial vlaue for the limit class, and answer it"

	| oneTooFar |
	limitClass := targetClass.
	(#('ProtoObject' 'Object' 'Behavior' 'ClassDescription' 'Class' 'ProtoObject class' 'Object class') includes: targetClass name asString) ifTrue: [^ targetClass].

	oneTooFar := (targetClass isKindOf: Metaclass)
		ifTrue:
			["use the fifth back from the superclass chain for Metaclasses, which is the immediate subclass of ProtoObject class.  Print <ProtoObject class allSuperclasses> to count them yourself."
			targetClass allSuperclasses at: (targetClass allSuperclasses size - 5)]
		ifFalse:
			[targetClass allSuperclasses at: targetClass allSuperclasses size].
	[limitClass superclass ~~ oneTooFar]
		whileTrue: [limitClass := limitClass superclass].
	^ limitClass! !

!Lexicon methodsFor: 'initialization' stamp: 'nice 12/27/2009 03:10'!
initListFrom: selectorCollection highlighting: aClass 
	"Make up the messageList with items from aClass in boldface.  Provide a final filtering in that only selectors whose implementations fall within my limitClass will be shown."

	
	messageList := OrderedCollection new.
	selectorCollection do: 
		[:selector | | item defClass |  defClass := aClass whichClassIncludesSelector: selector.
		(defClass notNil and: [defClass includesBehavior: self limitClass]) ifTrue:
			[item := selector, '     (' , defClass name , ')'.
			item := item asText.
			defClass == aClass ifTrue: [item allBold].
			"(self isThereAnOverrideOf: selector) ifTrue: [item addAttribute: TextEmphasis struckOut]."
			"The above has a germ of a good idea but could be very slow"
			messageList add: item]]! !

!Lexicon methodsFor: 'search' stamp: 'sw 3/20/2001 18:55'!
lastSearchString
	"Answer the last search string, initializing it to an empty string if it has not been initialized yet"

	^ currentQueryParameter ifNil: [currentQueryParameter := 'contents']! !

!Lexicon methodsFor: 'search' stamp: 'sw 4/12/2001 00:42'!
lastSearchString: aString
	"Make a note of the last string searched for in the receiver"

	currentQueryParameter := aString asString.
	currentQuery := #selectorName.
	autoSelectString := aString.
	self setMethodListFromSearchString.
	^ true! !

!Lexicon methodsFor: 'search' stamp: 'sw 3/20/2001 19:00'!
lastSendersSearchSelector
	"Answer the last senders search selector, initializing it to a default value if it does not already have a value"

	^ currentQueryParameter ifNil: [currentQueryParameter := #flag:]! !

!Lexicon methodsFor: 'limit class' stamp: 'sw 10/12/2001 21:30'!
limitClass
	"Answer the most generic class to show in the browser.  By default, we go all the way up to ProtoObject"

	^ limitClass ifNil: [self initialLimitClass]! !

!Lexicon methodsFor: 'limit class' stamp: 'sw 3/20/2001 13:07'!
limitClassString
	"Answer a string representing the current choice of most-generic-class-to-show"

	| most |
	(most := self limitClass) == ProtoObject
		ifTrue:	[^ 'All'].
	most == targetClass
		ifTrue:	[^ most name].
	^ 'Only through ', most name! !

!Lexicon methodsFor: 'limit class' stamp: 'sw 12/13/2000 06:49'!
limitClass: aClass
	"Set the most generic class to show as indicated"

	limitClass := aClass! !

!Lexicon methodsFor: 'initialization' stamp: 'fbs 4/14/2010 12:34'!
listFrame: bottomFraction fromLeft: leftFraction width: rightFraction
	^LayoutFrame new
		leftFraction: leftFraction offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: bottomFraction offset: self buttonHeight negated - 3! !

!Lexicon methodsFor: 'category list' stamp: 'topa 3/18/2015 10:51'!
mainCategoryListMenu: aMenu
	"Answer the menu for the category list"
	<categoryListMenu>
	^ aMenu addList: #('find...(f)' obtainNewSearchString); yourself! !

!Lexicon methodsFor: 'transition' stamp: 'sw 3/20/2001 12:11'!
maybeReselectClass: aClass selector: aSelector
	"The protocol or limitClass may have changed, so that there is a different categoryList.  Formerly, the given class and selector were selected; if it is possible to do so, reselect them now"

	aClass ifNil: [^ self].
	(currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: targetClass limitClass: limitClass)
		ifTrue:
			[self selectSelectorItsNaturalCategory: aSelector]! !

!Lexicon methodsFor: 'message list' stamp: 'mt 11/18/2015 13:41'!
messageHelpAt: anIndex
	"Not working due to text representation of message list."
	^ nil! !

!Lexicon methodsFor: 'message list' stamp: 'mt 11/18/2015 13:41'!
messageIconAt: anIndex
	"Not working due to text representation of message list."
	^ nil! !

!Lexicon methodsFor: 'basic operation' stamp: 'nk 2/14/2004 15:10'!
messageListIndex: anIndex
	"Set the message list index as indicated, and update the history list if appropriate"

	| newSelector current |
	current := self selectedMessageName.
	super messageListIndex: anIndex.
	anIndex = 0 ifTrue: [
		self editSelection: #newMessage.
		self contentsChanged].
	(newSelector := self selectedMessageName) ifNotNil: 
		[self updateSelectorsVisitedfrom: current to: newSelector]! !

!Lexicon methodsFor: 'message list menu' stamp: 'sw 4/20/2001 20:54'!
messageListKey: aChar from: view
	"Respond to a Command key"

	aChar == $f ifTrue: [^ self obtainNewSearchString].
	^ super messageListKey: aChar from: view! !

!Lexicon methodsFor: 'search' stamp: 'NS 12/12/2003 15:58'!
methodListFromSearchString: fragment
	"Answer a method list of methods whose selectors match the given fragment"

	|  aList searchFor |
	currentQueryParameter := fragment.
	currentQuery := #selectorName.
	autoSelectString := fragment.
	searchFor := fragment asString asLowercase withBlanksTrimmed.

	aList := targetClass allSelectors select:
		[:aSelector | currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: targetClass limitClass: limitClass].
	searchFor size > 0 ifTrue:
		[aList := aList select:
			[:aSelector | aSelector includesSubstring: searchFor caseSensitive: false]].
	^ aList asSortedArray
! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 11/21/2001 10:48'!
methodsWithInitials
	"Answer the list of method selectors within the scope of this tool whose time stamps begin with the initials designated by my currentQueryParameter"

	^ self methodsWithInitials: currentQueryParameter! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'NS 12/12/2003 15:58'!
methodsWithInitials: initials
	"Return a list of selectors representing methods whose timestamps have the given initials and which are in the protocol of this object and within the range dictated by my limitClass."

	| classToUse |
	classToUse := self targetObject ifNotNil: [self targetObject class] ifNil: [targetClass].  "In support of lightweight uniclasses"
	^ targetClass allSelectors select:
		[:aSelector | (currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: classToUse limitClass: limitClass) and:
			[Utilities doesMethod: aSelector forClass: classToUse bearInitials: initials]].

! !

!Lexicon methodsFor: 'senders' stamp: 'nice 3/24/2010 23:21'!
navigateToASender
	"Present the user with a list of senders of the currently-selected 
	message, and navigate to the chosen one"
	| selectorSet chosen aSelector |
	aSelector := self selectedMessageName.
	selectorSet := Set new.
	(self systemNavigation allCallsOn: aSelector)
		do: [:anItem | selectorSet add: anItem methodSymbol].
	selectorSet := selectorSet
				select: [:sel | currentVocabulary
						includesSelector: sel
						forInstance: self targetObject
						ofClass: targetClass
						limitClass: limitClass].
	selectorSet size = 0
		ifTrue: [^ Beeper beep].
	self okToChange
		ifFalse: [^ self].
	chosen := UIManager default chooseFrom: selectorSet asSortedArray values: selectorSet asSortedArray.
	chosen isEmptyOrNil
		ifFalse: [self displaySelector: chosen]! !

!Lexicon methodsFor: 'history' stamp: 'ul 11/15/2010 11:52'!
navigateToNextMethod
	"Navigate to the 'next' method in the current viewing sequence"

	| anIndex aSelector |
	self selectorsVisited size = 0 ifTrue: [^ self].
	anIndex := (aSelector := self selectedMessageName) notNil ifTrue: [selectorsVisited indexOf: aSelector ifAbsent: [selectorsVisited size]] ifFalse: [1].
	self selectedCategoryName == self class viewedCategoryName 
		ifTrue:
			[self selectWithinCurrentCategory: (selectorsVisited atWrap: (anIndex + 1))]
		ifFalse:
			[self displaySelector: (selectorsVisited atWrap: (anIndex + 1))]! !

!Lexicon methodsFor: 'history' stamp: 'ul 11/15/2010 11:52'!
navigateToPreviousMethod
	"Navigate to the 'previous' method in the current viewing sequence"

	| anIndex aSelector |
	self selectorsVisited size = 0 ifTrue: [^ self].
	anIndex := (aSelector := self selectedMessageName) notNil
		ifTrue: [selectorsVisited indexOf: aSelector ifAbsent: [selectorsVisited size]]
		ifFalse: [selectorsVisited size].
	self selectedCategoryName == self class viewedCategoryName 
		ifTrue:
			[self selectWithinCurrentCategory: (selectorsVisited atWrap: (anIndex - 1))]
		ifFalse:
			[self displaySelector: (selectorsVisited atWrap: (anIndex - 1))]! !

!Lexicon methodsFor: 'history' stamp: 'ar 8/6/2009 18:53'!
navigateToRecentMethod
	"Put up a menu of recent selectors visited and navigate to the one chosen"

	| visited aSelector |
	(visited := self selectorsVisited) size > 1 ifTrue:
		[visited := visited copyFrom: 1 to: (visited size min: 20).
		aSelector := UIManager default chooseFrom: visited values: visited 
			title: 'Recent methods visited in this browser'.
		aSelector isEmptyOrNil ifFalse: [self displaySelector: aSelector]]! !

!Lexicon methodsFor: 'category list' stamp: 'sw 12/28/2000 13:46'!
newCategoryPane
	"Formulate a category pane for insertion into the receiver's pane list"

	| aListMorph |
	aListMorph := PluggableListMorph on: self list: #categoryList
			selected: #categoryListIndex changeSelected: #categoryListIndex:
			menu: #categoryListMenu:shifted:
			keystroke: #categoryListKey:from:.
	aListMorph setNameTo: 'categoryList'.
	aListMorph menuTitleSelector: #categoryListMenuTitle.
	^ aListMorph! !

!Lexicon methodsFor: 'transition' stamp: 'sw 3/20/2001 00:41'!
noteAcceptanceOfCodeFor: newSelector
	"The user has submitted new code for the given selector; take a note of it.  NB that the selectors-changed list gets added to here, but is not currently used in the system."

	(self selectorsVisited includes: newSelector) ifFalse: [selectorsVisited add: newSelector].! !

!Lexicon methodsFor: 'search' stamp: 'ul 11/15/2010 11:52'!
obtainNewSearchString
	"Put up a box allowing the user to enter a fresh search string"

	| fragment |
	
	fragment := UIManager default request: 'type method name or fragment: ' initialAnswer: self currentQueryParameter.
	fragment ifNil: [^ self].
	(fragment := fragment copyWithout: $ ) size = 0  ifTrue: [^ self].
	currentQueryParameter := fragment.
	fragment := fragment asLowercase.
	currentQuery := #selectorName.
	self showQueryResultsCategory.
	self messageListIndex: 0! !

!Lexicon methodsFor: 'menu commands' stamp: 'cmm 10/16/2015 14:16'!
offerMenu
	"Offer a menu to the user, in response to the hitting of the menu button on the tool pane"

	| aMenu |
	aMenu := MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Lexicon'.
	aMenu addStayUpItem.
	aMenu addList: #(

		('vocabulary...' 			chooseVocabulary)
		('what to show...'			offerWhatToShowMenu)
		-
		('inst var refs (here)'		setLocalInstVarRefs)
		('inst var assignments (here)'		setLocalInstVarDefs)
		('class var refs (here)'		setLocalClassVarRefs)
		-

		('navigate to a sender...' 	navigateToASender)
		('recent...' 					navigateToRecentMethod)
		('show methods in current change set'
									showMethodsInCurrentChangeSet)
		('show methods with initials...'
									showMethodsWithInitials)
		-
		"('toggle search pane' 		toggleSearch)"

		-
		('browse full (b)' 			browseMethodFull)
		('browse hierarchy (h)'		classHierarchy)
		('browse protocol (p)'		browseFullProtocol)
		-
		('fileOut'					fileOutMessage)
		('printOut'					printOutMessage)
		-
		('senders of... (n)'			browseSendersOfMessages)
		('implementors of... (m)'		browseMessages)
		('versions (v)' 				browseVersions)
		('inheritance (i)'			methodHierarchy)
		-
		('references... (r)' 				browseVariableReferences)
		('assignments... (a)' 				browseVariableAssignments)
		-
		('more...'					shiftedYellowButtonActivity)).

	aMenu popUpInWorld: ActiveWorld! !

!Lexicon methodsFor: 'model glue' stamp: 'ul 11/15/2010 11:52'!
okayToAccept
	"Answer whether it is okay to accept the receiver's input"

	| ok aClass reply |
	(ok := super okayToAccept) ifTrue:
		[((aClass := self selectedClassOrMetaClass) ~~ targetClass) ifTrue:
			[reply := UIManager default chooseFrom: 
	{'okay, no problem'. 
	'cancel - let me reconsider'. 
	'compile into ', targetClass name, ' instead'.
	'compile into a new uniclass'} title:
'Caution!!  This would be
accepted into class ', aClass name, '.
Is that okay?' .
			reply = 1 ifTrue: [^ true].
			reply ~= 2 ifTrue:
				[self notYetImplemented].
			^ false]].
	^ ok! !

!Lexicon methodsFor: 'toolbuilder' stamp: 'ar 8/8/2009 00:30'!
openOnClass: aTargetClass inWorld: ignored showingSelector: aSelector

	^self openOnClass: aTargetClass showingSelector: aSelector! !

!Lexicon methodsFor: 'toolbuilder' stamp: 'ar 8/7/2009 23:24'!
openOnClass: aTargetClass showingSelector: aSelector

	currentVocabulary ifNil: [currentVocabulary := Vocabulary fullVocabulary].
	targetClass := aTargetClass.
	self initialLimitClass.
	
	self reformulateCategoryList.
	ToolBuilder open: self.
	self adjustWindowTitle.! !

!Lexicon methodsFor: 'transition' stamp: 'sw 12/11/2000 14:46'!
preserveSelectorIfPossibleSurrounding: aBlock
	"Make a note of the currently-selected method; perform aBlock and then attempt to reestablish that same method as the selected one in the new circumstances"

	| aClass aSelector |
	aClass := self selectedClassOrMetaClass.
	aSelector := self selectedMessageName.
	aBlock value.
	
	self hasSearchPane
		ifTrue:
			[self setMethodListFromSearchString]
		ifFalse:
			[self maybeReselectClass: aClass selector: aSelector]! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 7/23/2002 12:43'!
queryCharacterization
	"Answer a characterization of the most recent query"

	currentQuery == #selectorName
		ifTrue: [^ 'My methods whose names include "', self lastSearchString, '"'].
	currentQuery == #methodsWithInitials
		ifTrue: [^ 'My methods stamped with initials ', currentQueryParameter].
	currentQuery == #senders
		ifTrue: [^ 'My methods that send #', self lastSendersSearchSelector].
	currentQuery == #currentChangeSet
		ifTrue: [^ 'My methods in the current change set'].
	currentQuery == #instVarRefs
		ifTrue:	[^ 'My methods that refer to instance variable "', currentQueryParameter, '"'].
	currentQuery == #instVarDefs
		ifTrue:	[^ 'My methods that store into instance variable "', currentQueryParameter, '"'].
	currentQuery == #classVarRefs
		ifTrue:	[^ 'My methods that refer to class variable "', currentQueryParameter, '"'].
	^ 'Results of queries will show up here'! !

!Lexicon methodsFor: 'category list' stamp: 'sw 12/11/2000 14:47'!
reformulateCategoryList
	"Reformulate the category list"

	categoryList := nil.
	self categoryListIndex: 0.
	self changed: #categoryList.
	self contentsChanged! !

!Lexicon methodsFor: 'transition' stamp: 'sw 12/11/2000 02:00'!
reformulateList
	"Make the category list afresh, and reselect the current selector if appropriate"

	self preserveSelectorIfPossibleSurrounding:
		[super reformulateList.
		self categoryListIndex: categoryListIndex]! !

!Lexicon methodsFor: 'transition' stamp: 'sw 1/12/2001 00:33'!
reformulateListNoting: newSelector
	"A method has possibly been submitted for the receiver with newSelector as its selector; If the receiver has a way of reformulating its message list, here is a chance for it to do so"

	super reformulateListNoting: newSelector.
	newSelector ifNotNil:
		[self displaySelector: newSelector]! !

!Lexicon methodsFor: 'history' stamp: 'sw 3/19/2001 10:58'!
removeFromSelectorsVisited
	"Remove the currently-selected method from the active set"

	| aSelector |
	(aSelector := self selectedMessageName) ifNil: [^ self].
	self removeFromSelectorsVisited: aSelector.
	self chooseCategory: self class viewedCategoryName! !

!Lexicon methodsFor: 'history' stamp: 'sw 3/19/2001 07:43'!
removeFromSelectorsVisited: aSelector
	"remove aSelector from my history list"

	self selectorsVisited remove: aSelector ifAbsent: []! !

!Lexicon methodsFor: 'menu commands' stamp: 'sw 3/20/2001 22:23'!
removeMessage
	"Remove the selected message from the system."

	messageListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].

	super removeMessage.
	"my #reformulateList method, called from the super #removeMethod method, will however try to preserve the selection, so we take pains to clobber it by the below..."
	messageListIndex := 0.
	self changed: #messageList.
	self changed: #messageListIndex.
	contents := nil.
	self contentsChanged! !

!Lexicon methodsFor: 'morphic' stamp: 'cmm 2/13/2010 19:51'!
representsSameBrowseeAs: anotherModel
	^self hasUnacceptedEdits not
		and: [ anotherModel targetClass = self targetClass ]! !

!Lexicon methodsFor: 'transition' stamp: 'sw 12/19/2000 18:27'!
retainMethodSelectionWhileSwitchingToCategory: aCategoryName
	"retain method selection while switching the category-pane selection to show the category of the given name"

	| aSelectedName |
	aSelectedName := self selectedMessageName.
	self categoryListIndex: (categoryList indexOf: aCategoryName ifAbsent: [^ self]).
	aSelectedName ifNotNil: [self selectWithinCurrentCategory: aSelectedName]
! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'md 10/22/2003 16:14'!
seeAlso
	"Present a menu offering the selector of the currently selected message, as well as of all messages sent by it.  If the chosen selector is showable in the current browser, show it here, minding unsubmitted edits however"

	self selectImplementedMessageAndEvaluate:
		[:aSelector |
			((currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: targetClass limitClass: limitClass)  			 "i.e., is this aSelector available in this browser"
					and: [self okToChange])
				ifTrue:
					[self displaySelector: aSelector]
				ifFalse:
					[Beeper beep.  "SysttemNavigation new browseAllImplementorsOf: aSelector"]].
					"Initially I tried making this open an external implementors browser in this case, but later decided that the user model for this was unstable"! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'nb 6/17/2003 12:25'!
seeAlso: aSelector
	"If the requested selector is showable in the current browser, show it here, minding unsubmitted edits however"

	((currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: targetClass limitClass: limitClass)   "i.e., is aSelector available in this browser"
					and: [self okToChange])
		ifTrue:
			[self displaySelector: aSelector]
		ifFalse:
			[Beeper beep]! !

!Lexicon methodsFor: 'category list' stamp: 'sw 12/12/2000 19:38'!
selectedCategoryName
	"Answer the selected category name"

	^ categoryList ifNotNil:
		[categoryList at: categoryListIndex ifAbsent: [nil]]! !

!Lexicon methodsFor: 'selection' stamp: 'tk 9/15/2001 08:17'!
selectedClassOrMetaClass
	"Answer the currently selected class (or metaclass)."

	self setClassAndSelectorIn: [:c :s | ^c]! !

!Lexicon methodsFor: 'selection' stamp: 'ul 11/15/2010 11:53'!
selectedMessage
	"Answer the source method for the currently selected message."

	(categoryList notNil and: [(categoryListIndex isNil or: [categoryListIndex = 0])])
		ifTrue:
			[^ '---'].

	self setClassAndSelectorIn: [:class :selector | 
		class ifNil: [^ 'here would go the documentation for the protocol category, if any.'].

		self showingDecompile ifTrue: [^ self decompiledSourceIntoContents].
		self showingDocumentation ifTrue: [^ self commentContents].

		currentCompiledMethod := class compiledMethodAt: selector ifAbsent: [nil].
		^ self sourceStringPrettifiedAndDiffed asText makeSelectorBoldIn: class]! !

!Lexicon methodsFor: 'selection' stamp: 'ul 8/2/2011 20:02'!
selectImplementedMessageAndEvaluate: aBlock
	"Allow the user to choose one selector, chosen from the currently selected message's selector, as well as those of all messages sent by it, and evaluate aBlock on behalf of chosen selector.  If there is only one possible choice, simply make it; if there are multiple choices, put up a menu, and evaluate aBlock on behalf of the the chosen selector, doing nothing if the user declines to choose any.  In this variant, only selectors "

	| selector method messages |
	(selector := self selectedMessageName) ifNil: [^ self].
	method := (self selectedClassOrMetaClass ifNil: [^ self])
		compiledMethodAt: selector
		ifAbsent: [].
	(method isNil or: [(messages := method messages) size = 0])
		 ifTrue: [^ aBlock value: selector].
	(messages size = 1 and: [messages includes: selector])
		ifTrue:
			[^ aBlock value: selector].  "If only one item, there is no choice"

	messages := messages select: [:aSelector | currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: targetClass limitClass: limitClass].
	self systemNavigation 
		showMenuOf: messages
		withFirstItem: selector
		ifChosenDo: aBlock! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'nice 12/27/2009 03:10'!
selectorsChanged
	"Return a list of methods in the current change set (or satisfying some 
	other such criterion) that are in the protocol of this object"
	| aList targetedClass |
	targetedClass := self targetObject
				ifNil: [targetClass]
				ifNotNil: [self targetObject class].
	aList := OrderedCollection new.
	ChangeSet current methodChanges
		associationsDo: [:classChgAssoc | classChgAssoc value
				associationsDo: [:methodChgAssoc | | aClass | (methodChgAssoc value == #change
							or: [methodChgAssoc value == #add])
						ifTrue: [(aClass := targetedClass whichClassIncludesSelector: methodChgAssoc key)
								ifNotNil: [aClass name = classChgAssoc key
										ifTrue: [aList add: methodChgAssoc key]]]]].
	^ aList! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'nice 12/17/2013 23:25'!
selectorsDefiningInstVar
	"Return a list of methods that define a given inst var that are in the protocol of this object"

	| aList  |
	aList := OrderedCollection new.
	targetClass withAllSuperclassesDo:
		[:aClass | 
			aList addAll: (aClass whichMethodsStoreInto: currentQueryParameter asString)
		].
	^ aList! !

!Lexicon methodsFor: 'search' stamp: 'NS 12/12/2003 15:59'!
selectorsMatching
	"Anwer a list of selectors in the receiver that match the current search string"

	| fragment aList |
	fragment := self lastSearchString asLowercase.
	aList := targetClass allSelectors select:
		[:aSelector | (aSelector includesSubstring: fragment caseSensitive: false) and:
			[currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: targetClass limitClass: limitClass]].

	^ aList asSortedArray! !

!Lexicon methodsFor: 'category list' stamp: 'ael 12/19/2011 15:11'!
selectorsReferringToClassVar
	"Return a list of methods that refer to given class var that are in the 
	protocol of this object"
	| aList aClass nonMeta poolAssoc |
	nonMeta := targetClass theNonMetaClass.
	aClass := nonMeta classThatDefinesClassVariable: currentQueryParameter.
	aList := OrderedCollection new.
	poolAssoc := aClass classPool associationAt: currentQueryParameter asSymbol.
	(self systemNavigation allCallsOn: poolAssoc)
		do: [:elem | (nonMeta inheritsFrom: elem actualClass)
				ifTrue: [aList add: elem methodSymbol]].
	^ aList! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'nice 12/17/2013 23:25'!
selectorsReferringToInstVar
	"Return a list of methods that refer to a given inst var that are in the protocol of this object"

	| aList  |
	aList := OrderedCollection new.
	targetClass withAllSuperclassesDo: [:aClass | 
		aList addAll: (aClass whichSelectorsAccess: currentQueryParameter asString)
	].
	^ aList! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 11/21/2001 10:53'!
selectorsRetrieved
	"Anwer a list of selectors in the receiver that have been retrieved for the query category.  This protocol is used when reformulating a list after, say, a limitClass change"

	currentQuery == #classVarRefs ifTrue: [^ self selectorsReferringToClassVar].
	currentQuery == #currentChangeSet ifTrue: [^ self selectorsChanged].
	currentQuery == #instVarDefs ifTrue: [^ self selectorsDefiningInstVar].
	currentQuery == #instVarRefs ifTrue: [^ self selectorsReferringToInstVar].
	currentQuery == #methodsWithInitials ifTrue: [^ self methodsWithInitials].
	currentQuery == #selectorName ifTrue: [^ self selectorsMatching].
	currentQuery == #senders ifTrue: [^ self selectorsSendingSelectedSelector].

	^ #()! !

!Lexicon methodsFor: 'senders' stamp: 'nice 12/27/2009 03:10'!
selectorsSendingSelectedSelector
	"Assumes lastSendersSearchSelector is already set"
	| selectorSet |
	autoSelectString := (self lastSendersSearchSelector upTo: $:) asString.
	selectorSet := Set new.
	(self systemNavigation allCallsOn: self lastSendersSearchSelector)
		do: [:anItem | | sel cl | 
			sel := anItem methodSymbol.
			cl := anItem actualClass.
			((currentVocabulary
						includesSelector: sel
						forInstance: self targetObject
						ofClass: targetClass
						limitClass: limitClass)
					and: [targetClass includesBehavior: cl])
				ifTrue: [selectorSet add: sel]].
	^ selectorSet asSortedArray! !

!Lexicon methodsFor: 'history' stamp: 'sw 12/5/2000 16:27'!
selectorsVisited
	"Answer the list of selectors visited in this tool"

	^ selectorsVisited ifNil: [selectorsVisited := OrderedCollection new]! !

!Lexicon methodsFor: 'selection' stamp: 'sw 3/19/2001 12:14'!
selectSelectorItsNaturalCategory: aSelector
	"Make aSelector be the current selection of the receiver, with the category being its home category."

	| cat catIndex detectedItem |
	cat := self categoryOfSelector: aSelector.
	catIndex := categoryList indexOf: cat ifAbsent:
		["The method's own category is not seen in this browser; the method probably occurs in some other category not known directly to the class, but for now, we'll just use the all category"
		1].
	self categoryListIndex: catIndex.
	detectedItem := messageList detect:
		[:anItem | (anItem asString upTo: $ ) asSymbol == aSelector] ifNone: [^ self].
	self messageListIndex:  (messageList indexOf: detectedItem ifAbsent: [^ self])! !

!Lexicon methodsFor: 'category list' stamp: 'topa 10/19/2015 23:02:15'!
selectWithinCurrentCategoryIfPossible: aSelector
	"If the receiver's message list contains aSelector, navigate right to it without changing categories"
 
	| detectedItem messageIndex |
	aSelector ifNil: [^ self].
	detectedItem := messageList detect:
		[:anItem | (anItem asString copyUpTo: $ ) asSymbol == aSelector] ifNone: [^ self].
	messageIndex := messageList indexOf: detectedItem.
	self messageListIndex: messageIndex
! !

!Lexicon methodsFor: 'selection' stamp: 'sw 12/14/2000 13:48'!
selectWithinCurrentCategory: aSelector
	"If aSelector is one of the selectors seen in the current category, select it"

	| detectedItem |
	detectedItem := self messageList detect:
		[:anItem | (anItem asString upTo: $ ) asSymbol == aSelector] ifNone: [^ self].
	self messageListIndex:  (messageList indexOf: detectedItem ifAbsent: [^ self])! !

!Lexicon methodsFor: 'selection' stamp: 'tk 9/15/2001 08:14'!
setClassAndSelectorIn: csBlock
	"Decode strings of the form    <selectorName> (<className> [class])"


	self selection ifNil: [^ csBlock value: targetClass value: nil].
	^ super setClassAndSelectorIn: csBlock! !

!Lexicon methodsFor: 'limit class' stamp: 'rkrk 8/23/2009 05:29'!
setLimitClass: aClass
	"Set aClass as the limit class for this browser"

	| currentClass currentSelector |
	currentClass := self selectedClassOrMetaClass.
	currentSelector := self selectedMessageName.

	self limitClass: aClass.
	categoryList := nil.
	self categoryListIndex: 0.
	self changed: #categoryList.
	self changed: #methodList.
	self changed: #contents.
	self changed: #limitClassString.
	self adjustWindowTitle.
	self hasSearchPane
		ifTrue:
			[self setMethodListFromSearchString].

	self maybeReselectClass: currentClass selector: currentSelector

	! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 4/4/2001 00:15'!
setLocalClassVarRefs
	"Put up a list of the class variables in the viewed object, and when the user selects one, let the query results category show all the references to that class variable."

	| aName |

	(aName := targetClass theNonMetaClass chooseClassVarName) ifNil: [^ self].
	currentQuery := #classVarRefs.
	currentQueryParameter := aName.
	self showQueryResultsCategory! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'cmm 5/2/2011 09:49'!
setLocalInstVarDefs
	"Put up a list of the instance variables in the viewed object, and when the user seletcts one, let the query results category show all the references to that instance variable."

	| instVarToProbe |

	targetClass chooseVarThenDo:
		[:aName | instVarToProbe := aName].
	instVarToProbe isEmptyOrNil ifTrue: [^ self].
	currentQuery := #instVarDefs.
	currentQueryParameter := instVarToProbe.
	self showQueryResultsCategory! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'cmm 5/2/2011 09:49'!
setLocalInstVarRefs
	"Put up a list of the instance variables in the viewed object, and when the user seletcts one, let the query results category show all the references to that instance variable."

	| instVarToProbe |

	targetClass chooseVarThenDo:
		[:aName | instVarToProbe := aName].
	instVarToProbe isEmptyOrNil ifTrue: [^ self].
	currentQuery := #instVarRefs.
	currentQueryParameter := instVarToProbe.
	self showQueryResultsCategory! !

!Lexicon methodsFor: 'search' stamp: 'ul 11/15/2010 11:53'!
setMethodListFromSearchString
	"Set the method list of the receiver based on matches from the search string"

	| fragment aList |
	self okToChange ifFalse: [^ self].
	fragment := currentQueryParameter.
	fragment := fragment asString asLowercase withBlanksTrimmed.

	aList := targetClass allSelectors select:
		[:aSelector | currentVocabulary includesSelector: aSelector forInstance: self targetObject ofClass: targetClass limitClass: limitClass].
	fragment size > 0 ifTrue:
		[aList := aList select:
			[:aSelector | aSelector includesSubstring: fragment caseSensitive: false]].
	aList size = 0 ifTrue:
		[^ Beeper beep].
	self initListFrom: aList asSortedArray highlighting: targetClass.
	messageListIndex :=  messageListIndex min: messageList size.
	self changed: #messageList
! !

!Lexicon methodsFor: 'senders' stamp: 'rbb 3/1/2005 10:59'!
setSendersSearch
	"Put up a list of messages sent in the current message, find all methods 
	of the browsee which send the one the user chooses, and show that list 
	in the message-list pane, with the 'query results' item selected in the 
	category-list pane"
	| selectorSet aSelector aString |
	self selectedMessageName
		ifNil: [aString := UIManager default request: 'Type selector to search for' initialAnswer: 'flag:'.
			aString isEmptyOrNil
				ifTrue: [^ self].
			Symbol
				hasInterned: aString
				ifTrue: [:sel | aSelector := sel]]
		ifNotNil: [self
				selectMessageAndEvaluate: [:sel | aSelector := sel]].
	aSelector
		ifNil: [^ self].
	selectorSet := Set new.
	(self systemNavigation allCallsOn: aSelector)
		do: [:anItem | selectorSet add: anItem methodSymbol].
	selectorSet := selectorSet
				select: [:sel | currentVocabulary
						includesSelector: sel
						forInstance: self targetObject
						ofClass: targetClass
						limitClass: limitClass].
	selectorSet size > 0
		ifTrue: [currentQuery := #senders.
			currentQueryParameter := aSelector.
			self
				categoryListIndex: (categoryList indexOf: self class queryCategoryName).
			self messageListIndex: 0]! !

!Lexicon methodsFor: 'selection' stamp: 'sw 1/26/2001 19:42'!
setToShowSelector: aSelector
	"Set up the receiver so that it will show the given selector"

	| catName catIndex detectedItem messageIndex aList |
	catName := (aList := currentVocabulary categoriesContaining: aSelector  forClass: targetClass) size > 0
		ifTrue:
			[aList first]
		ifFalse:
			[self class allCategoryName].
	catIndex := categoryList indexOf: catName ifAbsent: [1].
	self categoryListIndex: catIndex.
	detectedItem := messageList detect:
		[:anItem | (anItem upTo: $ ) asString asSymbol == aSelector] ifNone: [^ self].
	messageIndex := messageList indexOf: detectedItem.
	self messageListIndex: messageIndex
! !

!Lexicon methodsFor: 'category list' stamp: 'nb 6/17/2003 12:25'!
showCategoriesPane
	"Show the categories pane instead of the search pane"

	| aPane |
	(aPane := self searchPane) ifNil: [^ Beeper beep].
	self containingWindow replacePane: aPane with: self newCategoryPane.
	categoryList := nil.
	self changed: #categoryList.
	self changed: #messageList! !

!Lexicon methodsFor: 'menu commands' stamp: 'sw 10/18/2001 08:10'!
showCategory
	"A revectoring blamable on history.  Not sent in the image, but grandfathered buttons may still send this."

	^ self showHomeCategory! !

!Lexicon methodsFor: 'menu commands' stamp: 'sw 10/8/2001 14:33'!
showHomeCategory
	"Continue to show the current selector, but show it within the context of its primary category"

	| aSelector |
	(aSelector := self selectedMessageName) ifNotNil:
		[self preserveSelectorIfPossibleSurrounding:
			[self setToShowSelector: aSelector]]! !

!Lexicon methodsFor: 'menu commands' stamp: 'sw 10/8/2001 14:34'!
showMainCategory
	"Continue to show the current selector, but show it within the context of its primary category.  Preserved for backward compatibility with pre-existing buttons."

	^ self showHomeCategory! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 11/21/2001 11:19'!
showMethodsInCurrentChangeSet
	"Set the current query to be for methods in the current change set"

	currentQuery := #currentChangeSet.
	autoSelectString := nil.
	self categoryListIndex: (categoryList indexOf: self class queryCategoryName).! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'rbb 3/1/2005 10:59'!
showMethodsWithInitials
	"Prompt the user for initials to scan for; then show, in the query-results category, all methods with those initials in their time stamps"

	| initials |
	initials := UIManager default request: 'whose initials? ' initialAnswer: Utilities authorInitials.
	initials isEmptyOrNil ifTrue: [^ self].
	self showMethodsWithInitials: initials


! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 11/21/2001 10:44'!
showMethodsWithInitials: initials
	"Make the current query be for methods stamped with the given initials"

	currentQuery := #methodsWithInitials.
	currentQueryParameter := initials.
	self showQueryResultsCategory.
	autoSelectString := nil.
	self changed: #messageList.
	self adjustWindowTitle
! !

!Lexicon methodsFor: 'within-tool queries' stamp: 'sw 3/20/2001 20:16'!
showQueryResultsCategory
	"Point the receiver at the query-results category and set the search string accordingly"

	autoSelectString := self currentQueryParameter.
	self categoryListIndex: (categoryList indexOf: self class queryCategoryName).
	self messageListIndex: 0! !

!Lexicon methodsFor: 'search' stamp: 'nb 6/17/2003 12:25'!
showSearchPane
	"Given that the receiver is showing the categories pane, replace that with a search pane.  Though there is a residual UI for obtaining this variant, it is obscure and the integrity of the protocol-category-browser when there is no categories pane is not necessarily assured at the moment."

	| aPane |
	(aPane := self categoriesPane) ifNil: [^ Beeper beep].
	self containingWindow replacePane: aPane with: self newSearchPane.
	categoryList := nil.
	self changed: #categoryList.
	self changed: #messageList! !

!Lexicon methodsFor: 'window title' stamp: 'sw 3/20/2001 12:18'!
startingWindowTitle
	"Answer the initial window title to apply"

	^ 'Vocabulary of ', targetClass nameForViewer! !

!Lexicon methodsFor: 'vocabulary' stamp: 'sw 1/26/2001 19:40'!
switchToVocabulary: aVocabulary
	"Make aVocabulary be the current one in the receiver"

	self preserveSelectorIfPossibleSurrounding:
		[self useVocabulary: aVocabulary.
		self reformulateCategoryList.
		self adjustWindowTitle]
! !

!Lexicon methodsFor: 'morphic' stamp: 'cmm 2/23/2002 16:03'!
targetClass

	^targetClass
! !

!Lexicon methodsFor: 'model glue' stamp: 'sw 3/20/2001 12:25'!
targetObject
	"Answer the object to which this tool is bound."

	^ nil! !

!Lexicon methodsFor: 'search' stamp: 'sw 12/11/2000 14:46'!
toggleSearch
	"Toggle the determination of whether a categories pane or a search pane shows"

	self hasSearchPane
		ifTrue:	[self showCategoriesPane]
		ifFalse:	[self showSearchPane]! !

!Lexicon methodsFor: 'history' stamp: 'sw 12/11/2000 08:49'!
updateSelectorsVisitedfrom: oldSelector to: newSelector
	"Update the list of selectors visited."

	newSelector == oldSelector ifTrue: [^ self].
	self selectorsVisited remove: newSelector ifAbsent: [].
		
	(selectorsVisited includes:  oldSelector)
		ifTrue:
			[selectorsVisited add: newSelector after: oldSelector]
		ifFalse:
			[selectorsVisited add: newSelector]
! !

!Lexicon methodsFor: 'vocabulary' stamp: 'sw 1/26/2001 19:37'!
useVocabulary: aVocabulary
	"Set up the receiver to use the given vocabulary"

	currentVocabulary := aVocabulary! !

!Lexicon methodsFor: 'toolbuilder' stamp: 'sw 1/24/2001 21:25'!
wantsAnnotationPane
	"This kind of browser always wants annotation panes, so answer true"

	^ true! !

!ProtocolBrowser class methodsFor: 'instance creation' stamp: 'di 7/13/97 15:15'!
openFullProtocolForClass: aClass 
	"Create and schedule a browser for the entire protocol of the class."
	"ProtocolBrowser openFullProtocolForClass: ProtocolBrowser."
	| aPBrowser label |
	aPBrowser := ProtocolBrowser new on: aClass.
	label := 'Entire protocol of: ', aClass name.
	self open: aPBrowser name: label! !

!ProtocolBrowser class methodsFor: 'instance creation' stamp: 'di 7/13/97 15:15'!
openSubProtocolForClass: aClass 
	"Create and schedule a browser for the entire protocol of the class."
	"ProtocolBrowser openSubProtocolForClass: ProtocolBrowser."
	| aPBrowser label |
	aPBrowser := ProtocolBrowser new onSubProtocolOf: aClass.
	label := 'Sub-protocol of: ', aClass name.
	self open: aPBrowser name: label! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:33'!
getList
	"Answer the receiver's message list."
	^ messageList! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'sw 1/28/2001 21:01'!
growable
	"Answer whether the receiver is subject to manual additions and deletions"

	^ false! !

!ProtocolBrowser methodsFor: 'private' stamp: 'nice 12/26/2009 20:38'!
initListFrom: selectorCollection highlighting: aClass 
	"Make up the messageList with items from aClass in boldface."
	messageList := OrderedCollection new.
	selectorCollection do: [ :selector |  
		| defClass item |
		defClass := aClass whichClassIncludesSelector: selector.
		item := selector, '     (' , defClass name , ')'.
		defClass == aClass ifTrue: [item := item asText allBold].
		messageList add: (
			MethodReference new
				setClass: defClass 
				methodSymbol: selector 
				stringVersion: item
		)
	].
	selectedClass := aClass.! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:33'!
list
	"Answer the receiver's message list."
	^ messageList! !

!ProtocolBrowser methodsFor: 'private' stamp: 'di 11/26/1999 19:39'!
onSubProtocolOf: aClass 
	"Initialize with the entire protocol for the class, aClass,
		but excluding those inherited from Object."
	| selectors |
	selectors := Set new.
	aClass withAllSuperclasses do:
		[:each | (each == Object or: [each == ProtoObject]) 
			ifFalse: [selectors addAll: each selectors]].
	self initListFrom: selectors asSortedCollection
		highlighting: aClass! !

!ProtocolBrowser methodsFor: 'private' stamp: 'di 7/13/97 16:26'!
on: aClass 
	"Initialize with the entire protocol for the class, aClass."
	self initListFrom: aClass allSelectors asSortedCollection
		highlighting: aClass! !

!ProtocolBrowser methodsFor: 'class list' stamp: 'nk 4/10/2001 08:16'!
selectedClassOrMetaClass
	^selectedClass! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:35'!
selector
	"Answer the receiver's selected selector."
	^ selectedSelector! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:35'!
selector: aString
	"Set the currently selected message selector to be aString."
	selectedSelector := aString.
	self changed: #selector! !

!ProtocolBrowser methodsFor: 'private' stamp: 'RAA 5/28/2001 11:07'!
setClassAndSelectorIn: csBlock
	"Decode strings of the form    <selectorName> (<className> [class])"

	| i classAndSelString selString sel |

	sel := self selection ifNil: [^ csBlock value: nil value: nil].
	(sel isKindOf: MethodReference) ifTrue: [
		sel setClassAndSelectorIn: csBlock
	] ifFalse: [
		selString := sel asString.
		i := selString indexOf: $(.
		"Rearrange to  <className> [class] <selectorName> , and use MessageSet"
		classAndSelString := (selString copyFrom: i + 1 to: selString size - 1) , ' ' ,
							(selString copyFrom: 1 to: i - 1) withoutTrailingBlanks.
		MessageSet parse: classAndSelString toClassAndSelector: csBlock.
	].
! !

!ProtocolBrowser methodsFor: 'accessing' stamp: 'di 7/13/97 16:35'!
setSelector: aString
	"Set the currently selected message selector to be aString."
	selectedSelector := aString! !

Object subclass: #ElementTranslation
	instanceVariableNames: 'wording helpMessage naturalLanguageSymbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!ElementTranslation methodsFor: 'access' stamp: 'sw 8/18/2004 22:12'!
helpMessage
	"Answer the helpMessage"

	^ helpMessage! !

Object subclass: #ObjectWithDocumentation
	instanceVariableNames: 'authoringStamp properties elementSymbol naturalLanguageTranslations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!ObjectWithDocumentation commentStamp: 'fbs 6/20/2013 17:25' prior: 0!
ObjectWithDocumentation - an abstract superclass for objects that allows maintenance of an authoring stamp, a body of documentation, and a properties dictionary.
The Properties implementation has not happened yet -- it would closely mirror the implementation of properties in the MorphExtension, for example.!

ObjectWithDocumentation subclass: #ElementCategory
	instanceVariableNames: 'categoryName keysInOrder elementDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!ElementCategory commentStamp: '<historical>' prior: 0!
ElementCategory
	
Contains a list of elements that affords keyed access but also has an inherent order.

Add items to the category by sending it elementAt:put:.
Obtain the elements in order by sending #elementsInOrder
Obtain the value of an element at a given key by sending #elementAt:!

!ElementCategory methodsFor: 'initialization' stamp: 'sw 3/30/2001 00:12'!
addCategoryItem: anItem
	"Add the item at the end, obtaining its key from itself (it must respond to #categoryName)"

	self elementAt: anItem categoryName put: anItem! !

!ElementCategory methodsFor: 'category name' stamp: 'sw 1/26/2001 22:45'!
categoryName
	"Answer the formal name of the category"

	^ categoryName! !

!ElementCategory methodsFor: 'category name' stamp: 'sw 1/26/2001 22:46'!
categoryName: aName
	"Set the category name"

	categoryName := aName! !

!ElementCategory methodsFor: 'initialization' stamp: 'sw 3/28/2001 19:46'!
clear
	"Clear the receiber's keysInOrder and elementDictionary"

	keysInOrder := OrderedCollection new.
	elementDictionary := IdentityDictionary new! !

!ElementCategory methodsFor: 'copying' stamp: 'sw 12/1/2000 22:46'!
copyFrom: donor
	"Copy the receiver's contents from the donor"

	keysInOrder := donor keysInOrder.
	elementDictionary := donor copyOfElementDictionary! !

!ElementCategory methodsFor: 'copying' stamp: 'sw 12/1/2000 22:46'!
copyOfElementDictionary
	"Answer a copy of the element dictionary"

	^ elementDictionary copy! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 12/1/2000 22:46'!
elementAt: aKey
	"Answer the element at the given key"

	^ elementDictionary at: aKey ifAbsent: [nil]! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 1/26/2001 22:54'!
elementAt: sym put: element
	"Add symbol at the end of my sorted list (unless it is already present), and put the element in the dictionary"

	(keysInOrder includes: sym) ifFalse: [keysInOrder add: sym].
	^ elementDictionary at: sym put: element! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 9/12/2001 22:59'!
elementSymbol
	"Answer the element symbol for the receiver.  Here, the categoryName dominates"

	^ categoryName! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 12/1/2000 22:47'!
elementsInOrder
	"Answer the elements in order"

	^ keysInOrder collect: [:aKey | elementDictionary at: aKey]! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 4/3/2001 11:06'!
fasterElementAt: sym put: element
	"Add symbol at the end of my sorted list and put the element in the dictionary.  This variant adds the key at the end of the keys list without checking whether it already exists."

	keysInOrder add: sym.
	^ elementDictionary at: sym put: element! !

!ElementCategory methodsFor: 'keys' stamp: 'sw 12/11/2000 15:36'!
includesKey: aKey
	"Answer whether the receiver's dictionary holds the given key"

	^ elementDictionary includesKey: aKey! !

!ElementCategory methodsFor: 'initialization' stamp: 'sw 3/28/2001 19:47'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self clear! !

!ElementCategory methodsFor: 'private' stamp: 'sw 8/6/2004 10:34'!
initWordingAndDocumentation
	"Initialize wording and documentation (helpMessage) for getters and setters"

	self wording: self categoryName! !

!ElementCategory methodsFor: 'keys' stamp: 'sw 12/1/2000 22:47'!
keysInOrder
	"Answer the keys in their sorted order"

	^ keysInOrder copy! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 1/26/2001 23:00'!
placeKey: key1 afterKey: key2
	"Place the first key after the second one in my keysInOrder ordering"

	keysInOrder remove: key1.
	keysInOrder add: key1 after: key2! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 1/26/2001 23:00'!
placeKey: key1 beforeKey: key2
	"Place the first key before the second one in my keysInOrder ordering"

	keysInOrder remove: key1.
	keysInOrder add: key1 before: key2! !

!ElementCategory methodsFor: 'copying' stamp: 'nice 10/1/2010 23:40'!
postCopy
	"Copy own instance variables."

	super postCopy.
	self copyFrom: self! !

!ElementCategory methodsFor: 'printing' stamp: 'sw 1/26/2001 22:47'!
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."

	super printOn: aStream.
	categoryName ifNotNil: [aStream nextPutAll: ' named ', categoryName asString]! !

!ElementCategory methodsFor: 'elements' stamp: 'sw 4/11/2001 20:08'!
removeElementAt: aKey
	"Remove the element at the given key"

	elementDictionary removeKey: aKey ifAbsent: [^ self].
	keysInOrder remove: aKey ifAbsent: []! !

!ElementCategory methodsFor: 'translation' stamp: 'dgd 12/4/2003 20:22'!
translated
	"answer the receiver translated to the current language"
	
	^ self class new categoryName: categoryName asString translated asSymbol! !

ObjectWithDocumentation subclass: #MethodInterface
	instanceVariableNames: 'selector argumentVariables resultSpecification receiverType attributeKeywords defaultStatus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!MethodInterface commentStamp: '<historical>' prior: 0!
A MethodInterface describes the interface for a single method.  The most generic form is not bound to any particular class or object but rather describes an idealized interface.

	selector					A symbol - the selector being described
	argumentSpecifications	A list of specifications for the formal arguments of the method
	resultSpecification 		A characterization of the return value of the method
	userLevel				
	attributeKeywords		A list of symbols, comprising keywords that the user wishes to
								see on the screen for this method
	defaultStatus			The status to apply to new instances of the class by default
							(#ticking, #paused, #normal, etc.)


!

!MethodInterface class methodsFor: 'utilities' stamp: 'sw 7/17/2001 19:08'!
firingInterface
	"Answer an instance of the receiver representing #fire"

	^ self new selector: #fire type: nil setter: nil! !

!MethodInterface class methodsFor: 'utilities' stamp: 'gk 3/1/2005 10:43'!
isNullMarker: aMarker
	"Answer true if aMarker is nil or is one of the symbols in #(none #nil unused missing) -- to service a variety of historical conventions"

	^ aMarker isNil or: [#(none #nil unused missing) includes: aMarker]

"
MethodInterface isNullMarker: nil
MethodInterface isNullMarker: #nil
MethodInterface isNullMarker: #none
MethodInterface isNullMarker: #znak
"! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 3/10/2001 00:38'!
argumentVariables
	"Answer the list of argumentVariables of the interface"

	^ argumentVariables ifNil: [argumentVariables := OrderedCollection new]! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 5/2/2001 21:19'!
argumentVariables: variableList
	"Set the argument variables"

	argumentVariables := variableList! !

!MethodInterface methodsFor: 'attribute keywords' stamp: 'sw 1/26/2001 16:31'!
attributeKeywords
	"Answer a list of attribute keywords associated with the receiver"

	^ attributeKeywords ifNil: [attributeKeywords := OrderedCollection new]! !

!MethodInterface methodsFor: 'access' stamp: 'sw 3/8/2001 16:29'!
companionSetterSelector
	"If there is a companion setter selector, anwer it, else answer nil"

	^ resultSpecification ifNotNil:
		[resultSpecification companionSetterSelector]! !

!MethodInterface methodsFor: 'initialization' stamp: 'mir 7/12/2004 19:36'!
conjuredUpFor: aSelector class: aClass
	"Initialize the receiver to have the given selector, obtaining whatever info one can from aClass.  This basically covers the situation where no formal definition has been made."

	| parts |
	self initializeFor: aSelector.
	self wording: aSelector.

	receiverType := #unknown.
	parts := aClass formalHeaderPartsFor: aSelector.
	argumentVariables := (1 to: selector numArgs) collect:
		[:anIndex | Variable new name: (parts at: (4 * anIndex)) type: #Object].
	parts last isEmptyOrNil ifFalse: [self documentation: parts last].
! !

!MethodInterface methodsFor: 'status' stamp: 'sw 1/23/2001 17:16'!
defaultStatus
	"Answer the receiver's default defaultStatus"

	^ defaultStatus! !

!MethodInterface methodsFor: 'status' stamp: 'sw 1/23/2001 17:00'!
defaultStatus: aStatus
	"Set the receiver's defaultStatus as indicated"

	defaultStatus := aStatus! !

!MethodInterface methodsFor: 'access' stamp: 'sw 9/13/2001 16:42'!
elementSymbol
	"Answer the element symbol, for the purposes of translation"

	^ selector! !

!MethodInterface methodsFor: 'attribute keywords' stamp: 'sw 1/26/2001 16:33'!
flagAttributes: attributeSymbolList
	"Mark the receiver has being flagged with all the symbols in the list provided"

	attributeSymbolList do: [:aSym | self flagAttribute: aSym]! !

!MethodInterface methodsFor: 'attribute keywords' stamp: 'sw 1/26/2001 16:32'!
flagAttribute: aSymbol
	"Mark the receiver as having the given category-keyword"

	(self attributeKeywords includes: aSymbol) ifFalse: [attributeKeywords add: aSymbol]! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 3/9/2001 17:00'!
initialize
	"Initialize the receiver"

	super initialize.
	attributeKeywords := OrderedCollection new.
	defaultStatus := #normal.
	argumentVariables := OrderedCollection new
! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 2/24/2001 00:34'!
initializeFor: aSelector
	"Initialize the receiver to have the given selector"

	selector := aSelector.
	attributeKeywords := OrderedCollection new.
	defaultStatus := #normal
! !

!MethodInterface methodsFor: 'initialization' stamp: 'mir 7/12/2004 19:40'!
initializeSetterFromEToySlotSpec: tuple
	"tuple holds an old etoy slot-item spec, of the form found in #additionsToViewerCategories methods.   Initialize the receiver to represent the getter of this item"

	selector := tuple ninth.
	self
		wording: ('set ', tuple second);
		helpMessage: ('setter for', tuple third).
	receiverType := #Player.
	argumentVariables := Array with: (Variable new variableType: tuple fourth)
	! !

!MethodInterface methodsFor: 'attribute keywords' stamp: 'sw 1/26/2001 16:34'!
isFlaggedAs: aSymbol
	"Answer whether the receiver is flagged with the given trait"

	^ self attributeKeywords includes: aSymbol! !

!MethodInterface methodsFor: 'printing' stamp: 'nk 8/20/2004 09:38'!
printOn: aStream
	"print the receiver on a stream.  Overridden to provide details about wording, selector, result type, and companion setter."

	super printOn: aStream.
	aStream nextPutAll: ' - wording: ';
		print: self wording;
		nextPutAll: ' selector: ';
		print: selector.
	self argumentVariables size > 0 ifTrue:
		[aStream nextPutAll: ' Arguments: '.
		argumentVariables doWithIndex:
			[:aVariable :anIndex | 
				aStream nextPutAll: 'argument #', anIndex printString, ' name = ', aVariable variableName asString, ', type = ', aVariable variableType]].
	resultSpecification ifNotNil:
		[aStream nextPutAll: ' result type = ', resultSpecification resultType asString.
		resultSpecification companionSetterSelector ifNotNil:
			[aStream nextPutAll: ' setter = ', resultSpecification companionSetterSelector asString]]
	! !

!MethodInterface methodsFor: 'access' stamp: 'sw 2/24/2001 12:04'!
receiverType
	"Answer the receiver type"

	^ receiverType ifNil: [receiverType := #unknown]! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 3/7/2001 13:05'!
receiverType: aType
	"set the receiver type.  Whether the receiverType earns its keep here is not yet well understood.  At the moment, this is unsent"

	receiverType := aType! !

!MethodInterface methodsFor: 'initialize-release' stamp: 'ar 3/3/2001 19:38'!
releaseCachedState
	"Sent by player"! !

!MethodInterface methodsFor: 'access' stamp: 'sw 3/10/2001 00:38'!
resultType
	"Answer the result type"

	^ resultSpecification
		ifNotNil:
			[resultSpecification type]
		ifNil:
			[#unknown]! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 10/23/2001 05:42'!
resultType: aType
	"Set the receiver's resultSpecification to be a ResultType of the given type"

	resultSpecification := ResultSpecification new.
	resultSpecification resultType: aType! !

!MethodInterface methodsFor: 'access' stamp: 'sw 1/23/2001 18:37'!
selector
	"Answer the receiver's selector"

	^ selector! !

!MethodInterface methodsFor: 'attribute keywords' stamp: 'sw 5/4/2001 07:02'!
selector: aSelector type: aType setter: aSetter
	"Set the receiver's fields as indicated.  Values of nil or #none for the result type and the setter indicate that there is none"

	selector := aSelector.
	(MethodInterface isNullMarker: aType) ifFalse:
		[resultSpecification := ResultSpecification new.
		resultSpecification resultType: aType.
		(MethodInterface isNullMarker: aSetter) ifFalse:
			[resultSpecification companionSetterSelector: aSetter]]! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 5/26/2001 22:59'!
setNotToRefresh
	"Set the receiver up not to do periodic refresh."

	resultSpecification ifNotNil: [resultSpecification refetchFrequency: nil]! !

!MethodInterface methodsFor: 'initialization' stamp: 'sw 5/3/2001 15:59'!
setToRefetch
	"Set the receiver up to expect a refetch, assuming it has a result specification"

	resultSpecification ifNotNil: [resultSpecification refetchFrequency: 1]! !

!MethodInterface methodsFor: 'access' stamp: 'sw 3/9/2001 17:02'!
typeForArgumentNumber: anArgumentNumber
	"Answer the data type for the given argument number"

	| aVariable |
	aVariable := self argumentVariables at: anArgumentNumber.
	^ aVariable variableType! !

!MethodInterface methodsFor: 'access' stamp: 'sw 5/3/2001 01:10'!
wantsReadoutInViewer
	"Answer whether the method represented by the receiver is one which should have a readout in a viewer"

	^ resultSpecification notNil and:
		[resultSpecification refetchFrequency notNil]! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'mir 7/12/2004 21:21'!
documentation
	"Answer the receiver's documentation"

	^self helpMessage! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'mir 7/12/2004 19:33'!
documentation: somethingUsefulHopefully
	"Set the receiver's documentation, in the current langauge"

	self helpMessage: somethingUsefulHopefully! !

!ObjectWithDocumentation methodsFor: 'documentation' stamp: 'nice 8/27/2010 20:52'!
editDescription
	"Allow the user to see and edit the documentation for this object"
	| reply helpMessage |
	helpMessage := self documentation ifNil: [String new].
	reply := UIManager default
				multiLineRequest: 'Kindly edit the description' translated
				centerAt: Sensor cursorPoint
				initialAnswer: helpMessage
				answerHeight: 200.
	reply isEmptyOrNil
		ifFalse: [self documentation: reply]! !

!ObjectWithDocumentation methodsFor: 'miscellaneous' stamp: 'sw 9/12/2001 23:03'!
elementSymbol
	"Answer the receiver's element symbol"

	^ elementSymbol! !

!ObjectWithDocumentation methodsFor: 'private' stamp: 'mir 7/12/2004 21:28'!
getterSetterHelpMessage
	"Returns a helpMessage that has been computed previously and needs to be translated and then formatted with the elementSymbol.
	'get value of {1}' translated format: {elSym}"

	^(self propertyAt: #getterSetterHelpMessage ifAbsent: [^nil])
		translated format: {self elementSymbol}! !

!ObjectWithDocumentation methodsFor: 'private' stamp: 'mir 7/12/2004 21:29'!
getterSetterHelpMessage: aString
	"Sets a helpMessage that needs to be translated and then formatted with the elementSymbol.
	'get value of {1}' translated format: {elSym}"

	self propertyAt: #getterSetterHelpMessage put: aString! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'bf 9/17/2010 15:22'!
helpMessage
	"Check if there is a getterSetterHelpMessage. 
	Otherwise try the normal help message or return nil."

	^ self getterSetterHelpMessage
		ifNil: [(self propertyAt: #helpMessage ifAbsent:
			[self legacyHelpMessage ifNil: [^ nil]]) translatedInDomain: 'Etoys-Tiles']! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'mir 7/12/2004 19:32'!
helpMessage: somethingUsefulHopefully
	"Set the receiver's documentation, in the current langauge"

	self propertyAt: #helpMessage put: somethingUsefulHopefully! !

!ObjectWithDocumentation methodsFor: 'initialization' stamp: 'sw 1/29/2001 09:28'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	authoringStamp := Utilities changeStampPerSe
! !

!ObjectWithDocumentation methodsFor: 'private' stamp: 'mir 7/12/2004 21:31'!
initWordingAndDocumentation
	"Initialize wording and documentation (helpMessage) for getters and setters"

	| elSym |
	elSym := self elementSymbol.
	elSym
		ifNil: [^self].

	((elSym beginsWith: 'get')
		and: [elSym size > 3])
		ifTrue: [
			self wording: (elSym allButFirst: 3) withFirstCharacterDownshifted.
			self getterSetterHelpMessage: 'get value of {1}']
		ifFalse: [
			((elSym beginsWith: 'set')
				and: [elSym size > 4])
				ifTrue: [
					self wording: (elSym allButFirst: 3) withFirstCharacterDownshifted.
					self getterSetterHelpMessage: 'set value of {1}']]! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'sw 8/18/2004 22:11'!
legacyHelpMessage
	"If I have a help message stashed in my legacy naturalTranslations slot, answer its translated rendition, else answer nil.  If I *do* come across a legacy help message, transfer it to my properties dictionary."

	| untranslated |
	naturalLanguageTranslations isEmptyOrNil  "only in legacy (pre-3.8) projects"
		ifTrue: [^ nil].
	untranslated := naturalLanguageTranslations first helpMessage ifNil: [^ nil].
	self propertyAt: #helpMessage put: untranslated.
	naturalLanguageTranslations removeFirst.
	naturalLanguageTranslations isEmpty ifTrue: [naturalLanguageTranslations := nil].
	^ untranslated translated! !

!ObjectWithDocumentation methodsFor: 'migration' stamp: 'mir 7/12/2004 23:45'!
migrateWordAndHelpMessage
	"Migrate the English wording and help message to the new structure"

	| englishElement |
	self initWordingAndDocumentation.
	(self properties includes: #wording)
		ifFalse: [
			englishElement := self naturalLanguageTranslations
				detect: [:each | each language == #English] ifNone: [^nil].
			self wording: englishElement wording.
			self helpMessage: englishElement helpMessage]! !

!ObjectWithDocumentation methodsFor: 'miscellaneous' stamp: 'mir 7/12/2004 23:20'!
naturalLanguageTranslations
	^naturalLanguageTranslations ifNil: [OrderedCollection new]! !

!ObjectWithDocumentation methodsFor: 'private' stamp: 'mir 7/12/2004 19:30'!
properties
	^properties ifNil: [properties := Dictionary new]! !

!ObjectWithDocumentation methodsFor: 'private' stamp: 'mir 7/12/2004 19:30'!
propertyAt: key
	^self propertyAt: key ifAbsent: [nil]! !

!ObjectWithDocumentation methodsFor: 'private' stamp: 'mir 7/12/2004 19:29'!
propertyAt: key ifAbsent: aBlock
	^properties
		ifNil: aBlock
		ifNotNil: [properties at: key ifAbsent: aBlock]! !

!ObjectWithDocumentation methodsFor: 'private' stamp: 'mir 7/12/2004 19:29'!
propertyAt: key put: aValue
	self properties at: key put: aValue! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'mir 7/12/2004 23:57'!
untranslatedHelpMessage
	"Check if there is a getterSetterHelpMessage. 
	Otherwise try the normal help message or return nil."

	^(self propertyAt: #getterSetterHelpMessage ifAbsent: [nil])
		ifNil: [(self propertyAt: #helpMessage ifAbsent: [nil])]! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'mir 7/12/2004 23:56'!
untranslatedWording
	"Answer the receiver's wording"

	^self propertyAt: #wording ifAbsent: [nil]! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'Richo 8/16/2010 22:27'!
wording
	"Answer the receiver's wording"

	| wording |
	(wording := self propertyAt: #wording ifAbsent: [nil])
		ifNotNil: [^wording translatedInDomain: 'Etoys-Tiles' or: 'Etoys'].

	self initWordingAndDocumentation.
	^self propertyAt: #wording ifAbsent: ['']! !

!ObjectWithDocumentation methodsFor: 'accessing' stamp: 'mir 7/12/2004 21:39'!
wording: aString
	"Set the receiver's wording, in the current langauge"

	self propertyAt: #wording put: aString! !

ObjectWithDocumentation subclass: #ResultSpecification
	instanceVariableNames: 'type companionSetterSelector refetchFrequency'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!ResultSpecification methodsFor: 'companion setter' stamp: 'sw 2/27/2001 09:15'!
companionSetterSelector
	"Answer the companion setter, nil if none"

	^ companionSetterSelector! !

!ResultSpecification methodsFor: 'companion setter' stamp: 'sw 12/11/2000 14:34'!
companionSetterSelector: aSetterSelector
	"Set the receiver's companionSetterSelector as indicated"

	companionSetterSelector := aSetterSelector! !

!ResultSpecification methodsFor: 'refetch' stamp: 'sw 5/3/2001 00:07'!
refetchFrequency
	"Answer the frequency with which the receiver should be refetched by a readout polling values from it, as in a Viewer.  Answer nil if not ever to be refetched automatically"

	^ refetchFrequency! !

!ResultSpecification methodsFor: 'refetch' stamp: 'sw 5/3/2001 00:29'!
refetchFrequency: aFrequency
	"Set the refetch frequency"

	refetchFrequency := aFrequency! !

!ResultSpecification methodsFor: 'result type' stamp: 'sw 2/27/2001 09:14'!
resultType
	"Answer the reciever's result type"

	^ type! !

!ResultSpecification methodsFor: 'result type' stamp: 'sw 2/24/2001 12:11'!
resultType: aType
	"Set the receiver's resultType as specified"

	type := aType! !

!ResultSpecification methodsFor: 'result type' stamp: 'sw 2/24/2001 12:12'!
type
	"Answer the reciever's type"

	^ type! !

ObjectWithDocumentation subclass: #Variable
	instanceVariableNames: 'defaultValue floatPrecision variableName variableType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!Variable methodsFor: 'value' stamp: 'sw 12/11/2000 14:06'!
defaultValue
	"Answer the default value to be supplied to the receiver"

	^ defaultValue! !

!Variable methodsFor: 'name' stamp: 'sw 3/12/2001 12:40'!
name: aName type: aType
	"Set the variable's name and type as indicated"

	variableName := aName.
	variableType := aType! !

!Variable methodsFor: 'name' stamp: 'sw 9/30/2004 05:02'!
printOn: aStream
	"Print the receiver on the stream"

	super printOn: aStream.
	aStream nextPutAll: ' named ', (self variableName ifNil: ['<unnamed>']), ' type = ', variableType printString, ' default val = ', defaultValue printString! !

!Variable methodsFor: 'value' stamp: 'tk 9/19/2001 09:09'!
sample
	"The closest we can come to an object for our type"

	| ty clsName |
	self defaultValue ifNotNil: [^ self defaultValue].
	ty := self variableType.
	"How translate a type like #player into a class?"
	clsName := ty asString.
	clsName at: 1 put: (clsName first asUppercase).
	clsName := clsName asSymbol.
	(Smalltalk includesKey: clsName) ifFalse: [self error: 'What type is this?'. ^ 5].
	^ (Smalltalk at: clsName) initializedInstance! !

!Variable methodsFor: 'name' stamp: 'sw 12/11/2000 14:07'!
variableName
	"Answer the variable name of the receiver"

	^ variableName! !

!Variable methodsFor: 'type' stamp: 'sw 12/11/2000 14:06'!
variableType
	"Anser the variable type of the receiver"

	^ variableType! !

!Variable methodsFor: 'type' stamp: 'sw 12/11/2000 14:06'!
variableType: aType
	"Set the receiver's variable type as requested"

	variableType := aType! !

ObjectWithDocumentation subclass: #Vocabulary
	instanceVariableNames: 'vocabularyName categories methodInterfaces object limitClass translationTable'
	classVariableNames: 'AllStandardVocabularies LanguageSymbols LanguageTable'
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!Vocabulary commentStamp: '<historical>' prior: 0!
Vocabulary

vocabularyName	a Symbol -- the formal name by which this vocabulary is known.
categories			a list of ElementCategory objects: the categories that comprise the vocabulary
methodInterfaces 	an IdentityDictionary; keys are method selectors, values are MethodInterfaces

A Vocabulary can be either be *abstract*, i.e. associated with a *class*, or it can be *bound to an instance*.  The form that is bound to an instance is still in its infancy however.

object				in the *bound* form, an actual object is associated with the vocabulary
limitClass			in the *bound* form, an actual limit class is assocaited with the vocabulary

AllMethodInterfaces	This class variable is available to hold on to all method interfaces
						defined in the system, regardless of class.  Not deployed in 
						the first version of this code to be circulated externally.

AllVocabularies		A dictionary associating symbols with actual abstract vocabulary instances

------
Hints on making a vocabulary for a new foreign language.  You need build a method like #addGermanVocabulary.  Execute

	Transcript show: Vocabulary eToyVocabulary strings.

and copy the text from the transcript to the method you are building.
	A cheap trick to get started is to use a web site that translates.  Here is how.  For an entry like:  

(clearTurtleTrails			'clear pen trails'			'Clear all the pen trails in the interior.')

	substitute exclamation points for single quotes by using Alt-shift-j.  Most of the statements are imperatives.

(clearOwnersPenTrails !!clear all pen trails!! !!clear all pen trails in my containing play field!!) 

	This translates to

(ClearOwnersPenTrails!! reinigt allen Kugelschreiber verfolgt!! !! Reinigt allen Kugelschreiber Pfade in meinem enthaltenden Spiel Feld sind!!) 

	Which is laughable, and will certainly stimulate the author to improve it.
------!

Vocabulary subclass: #DataType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!DataType commentStamp: 'sw 8/22/2002 15:01' prior: 0!
A Vocabulary representing typed data.!

DataType subclass: #BooleanType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!BooleanType commentStamp: 'sw 1/5/2005 22:15' prior: 0!
A data type representing Boolean values, i.e., true or false.!

!BooleanType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:23'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #Boolean! !

!BooleanType methodsFor: 'initial value' stamp: 'sw 9/27/2001 17:20'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"
	^ true! !

!BooleanType methodsFor: 'tiles' stamp: 'yo 2/18/2005 16:39'!
setFormatForDisplayer: aDisplayer
	"Set up the displayer to have the right format characteristics"

	aDisplayer useSymbolFormat.
	aDisplayer growable: true
! !

DataType subclass: #ColorType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!ColorType commentStamp: 'sw 1/5/2005 22:15' prior: 0!
A data type representing a Color value.!

!ColorType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:23'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #Color.! !

!ColorType methodsFor: 'initial value' stamp: 'sw 9/27/2001 17:28'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ Color random! !

!DataType methodsFor: 'initial value' stamp: 'sw 9/26/2001 12:00'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ 'no value'! !

!DataType methodsFor: 'queries' stamp: 'mir 7/15/2004 10:34'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	"^ (self class == DataType) not"  "i.e. subclasses yes, myself no"
	"Assuming this is an abstract class"
	^true! !

DataType subclass: #FullVocabulary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!FullVocabulary commentStamp: '<historical>' prior: 0!
The vocabulary that it all-encompassing.  Its categories consist of the union of all categories of a class and all its superclasses.  The methods in each category consist of those with selectors that are associated with that category.!

!FullVocabulary methodsFor: 'method list' stamp: 'sw 12/12/2000 12:22'!
allMethodsInCategory: categoryName forInstance: anObject ofClass: aClass
	"Answer a list of all methods which are in the given category, on behalf of anObject"

	| classToUse |
	classToUse := aClass ifNil: [anObject class].
	^ classToUse allMethodsInCategory: categoryName! !

!FullVocabulary methodsFor: 'queries' stamp: 'stephaneducasse 2/4/2006 20:39'!
categoriesContaining: aSelector forClass: aTargetClass 
	"Answer a list of category names (all symbols) of categories that contain 
	the given selector for the target object. Initially, this just returns one."
	| classDefiningSelector catName |
	classDefiningSelector := aTargetClass whichClassIncludesSelector: aSelector.
	classDefiningSelector
		ifNil: [^ OrderedCollection new].
	catName := classDefiningSelector whichCategoryIncludesSelector: aSelector.
	^ OrderedCollection with: catName! !

!FullVocabulary methodsFor: 'category list' stamp: 'sw 12/13/2000 17:34'!
categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass
	"Answer the category list for the given object, considering only code implemented in mostGeneric and lower (or higher, depending on which way you're facing"

	| classToUse |
	classToUse := anObject ifNil: [aClass] ifNotNil: [anObject class].
	^ mostGenericClass == classToUse
		ifTrue:
			[mostGenericClass organization categories]
		ifFalse:
			[classToUse allMethodCategoriesIntegratedThrough: mostGenericClass]! !

!FullVocabulary methodsFor: 'queries' stamp: 'stephaneducasse 2/4/2006 20:39'!
categoryWithNameIn: categoryNames thatIncludesSelector: aSelector forInstance: targetInstance ofClass: targetClass 
	"Answer the name of a category, from among the provided 
	categoryNames, which defines the selector for the given class. Here, if 
	the category designated by the implementing class is acceptable it is the 
	one returned"
	| aClass catName result |
	(aClass := targetClass whichClassIncludesSelector: aSelector)
		ifNotNil: [(categoryNames includes: (catName := aClass whichCategoryIncludesSelector: aSelector))
				ifTrue: [catName ~~ #'as yet unclassified'
						ifTrue: [^ catName]]].
	result := super
				categoryWithNameIn: categoryNames
				thatIncludesSelector: aSelector
				forInstance: targetInstance
				ofClass: aClass.
	^ result
		ifNil: [#'as yet unclassified']! !

!FullVocabulary methodsFor: 'queries' stamp: 'sw 12/12/2000 06:06'!
encompassesAPriori: aClass
	"Answer whether an object, by its very nature, is one that the receiver embraces"

	^ true! !

!FullVocabulary methodsFor: 'queries' stamp: 'sw 3/20/2001 15:42'!
includesDefinitionForSelector: aSelector
	"Answer whether the given selector is known to the vocabulary.  Unsent at the moment, may disappear."

	^ true! !

!FullVocabulary methodsFor: 'queries' stamp: 'sw 12/1/2000 21:57'!
includesSelector: aSelector
	"Answer whether the given selector is known to the vocabulary"

	^ true! !

!FullVocabulary methodsFor: 'queries' stamp: 'sw 12/14/2000 06:00'!
includesSelector: aSelector forInstance: anInstance ofClass: aTargetClass limitClass: mostGenericClass
	"Answer whether the vocabulary includes the given selector for the given class, only considering method implementations in mostGenericClass and lower"

	| classToUse aClass |
	classToUse := self classToUseFromInstance: anInstance ofClass: aTargetClass.
	^ (aClass := classToUse whichClassIncludesSelector: aSelector)
		ifNil:
			[false]
		ifNotNil:
			[aClass includesBehavior: mostGenericClass]! !

!FullVocabulary methodsFor: 'initialization' stamp: 'sw 9/25/2001 21:52'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')
Vocabulary initialize
"

	super initialize.
	vocabularyName := #Object.
	self documentation: '"Object" is all-encompassing vocabulary that embraces all methods understood by an object'.
	self rigAFewCategories! !

!FullVocabulary methodsFor: 'queries' stamp: 'sw 9/27/2001 03:28'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^ false! !

!FullVocabulary methodsFor: 'initialization' stamp: 'nice 12/26/2009 10:13'!
rigAFewCategories
	"Formerly used to rig generic categories, now seemingly disfunctional and in abeyance"

	true ifTrue: [^ self].

	self flag: #deferred.
"Vocabulary fullVocabulary rigAFewCategories "
	#(	(accessing	'Generally holds methods to read and write instance variables')
		(initialization	'messages typically sent when an object is created, to set up its initial state'))

		do:
			[:pair |
				| aMethodCategory |
				aMethodCategory := ElementCategory new categoryName: pair first.
				aMethodCategory documentation: pair second.
				self addCategory: aMethodCategory]! !

FullVocabulary subclass: #ScreenedVocabulary
	instanceVariableNames: 'methodScreeningBlock categoryScreeningBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Kernel'!

!ScreenedVocabulary methodsFor: 'enumeration' stamp: 'sw 1/5/2001 06:55'!
allMethodsInCategory: categoryName forInstance: anObject ofClass: aClass
	"Answer a list of all methods in the vocabulary which are in the given category, on behalf of the given class and object"

	^ (super allMethodsInCategory: categoryName forInstance: anObject ofClass: aClass) select:
		[:aSelector | self includesSelector: aSelector]! !

!ScreenedVocabulary methodsFor: 'enumeration' stamp: 'ul 8/2/2011 20:09'!
categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass
	"Answer the category list for the given object/class, considering only code implemented in mostGenericClass and lower"

	^(super categoryListForInstance: anObject ofClass: aClass limitClass: mostGenericClass)
		select: categoryScreeningBlock! !

!ScreenedVocabulary methodsFor: 'initialization' stamp: 'sw 12/14/2000 13:58'!
categoryScreeningBlock: aBlock
	"Set the receiver's categoryScreeningBlock to the block provided"

	categoryScreeningBlock := aBlock! !

!ScreenedVocabulary methodsFor: 'queries' stamp: 'sw 12/14/2000 14:06'!
includesSelector: aSelector
	"Answer whether the given selector is known to the vocabulary"

	^ methodScreeningBlock value: aSelector! !

!ScreenedVocabulary methodsFor: 'queries' stamp: 'sw 12/14/2000 06:01'!
includesSelector: aSelector forInstance: anInstance ofClass: aTargetClass limitClass: mostGenericClass
	"Answer whether the vocabulary includes the given selector for the given object, only considering method implementations in mostGenericClass and lower"

	^ (super includesSelector: aSelector forInstance: anInstance ofClass: aTargetClass limitClass: mostGenericClass) and:
		[self includesSelector: aSelector]! !

!ScreenedVocabulary methodsFor: 'initialization' stamp: 'sw 12/4/2000 04:40'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	vocabularyName :=  #Public.
	self documentation: '"Public" is vocabulary that excludes categories that start with "private" and methods that start with "private" or "pvt"'! !

!ScreenedVocabulary methodsFor: 'initialization' stamp: 'sw 12/14/2000 13:57'!
methodScreeningBlock: aBlock
	"Set the receiver's methodScreeningBlock to the block provided"

	methodScreeningBlock := aBlock! !

DataType subclass: #GraphicType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!GraphicType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:24'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #Graphic.! !

!GraphicType methodsFor: 'initial value' stamp: 'sw 9/27/2001 17:29'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ ScriptingSystem formAtKey: #PaintTab! !

DataType subclass: #MenuType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!MenuType commentStamp: 'sw 1/6/2005 03:45' prior: 0!
A type associated with menu-item values.  An imperfect thing thus far, only usable in the doMenuItem etoy scripting phrase.!

!MenuType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:24'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #Menu! !

!MenuType methodsFor: 'tiles' stamp: 'sw 1/5/2005 22:27'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^false! !

DataType subclass: #NumberType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!NumberType commentStamp: 'sw 10/3/2002 02:18' prior: 0!
NumberType is a data type representing a numeric value.!

!NumberType methodsFor: 'initialization' stamp: 'nice 12/26/2009 18:47'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	"Vocabulary replaceNumberVocabulary"
	"Vocabulary addVocabulary: Vocabulary newNumberVocabulary"

	self vocabularyName: #Number.
	self documentation: 'Numbers are things that can do arithmetic, have their magnitudes compared, etc.'.

#((comparing				'Determining which of two numbers is larger'
		(= < > <= >= ~= ~~))
(arithmetic 				'Basic numeric operation'
		(* + - / // \\ abs negated quo: rem:))
(testing 					'Testing a number'
		(even isDivisibleBy: negative odd positive sign))
(#'mathematical functions'	'Trigonometric and exponential functions'
		(cos exp ln log log: raisedTo: sin sqrt squared tan raisedToInteger:))
(converting 				'Converting a number to another form'
		(@ asInteger asPoint degreesToRadians radiansToDegrees asSmallAngleDegrees asSmallPositiveDegrees))
(#'truncation and round off' 'Making a real number (with a decimal point) into an integer'
		(ceiling floor roundTo: roundUpTo: rounded truncateTo: truncated))
) do:

		[:item | 
			| aMethodCategory |
			aMethodCategory := ElementCategory new categoryName: item first.
			aMethodCategory documentation: item second.
			item third do:
				[:aSelector | 
					| aMethodInterface |
					aMethodInterface := MethodInterface new conjuredUpFor: aSelector class: (Number whichClassIncludesSelector: aSelector).
					aMethodInterface argumentVariables do:
							[:var | var variableType: #Number].

					(#(* + - / // \\ abs negated quo: rem:
						cos exp ln log log: raisedTo: sin sqrt squared tan raisedToInteger:
						asInteger degreesToRadians radiansToDegrees asSmallAngleDegrees asSmallPositiveDegrees)
							includes: aSelector) ifTrue:
								[aMethodInterface resultType: #Number].

					(#( @  asPoint ) includes: aSelector) ifTrue:
						[aMethodInterface resultType: #Point].

					(#(= < > <= >= ~= ~~ even isDivisibleBy: negative odd positive) includes: aSelector) ifTrue:
						[aMethodInterface resultType: #Boolean].

					aMethodInterface setNotToRefresh.  
					self atKey: aSelector putMethodInterface: aMethodInterface.
					aMethodCategory elementAt: aSelector put: aMethodInterface].
			self addCategory: aMethodCategory].

"
(('truncation and round off' ceiling detentBy:atMultiplesOf:snap: floor roundTo: roundUpTo: rounded truncateTo: truncated)
('testing' basicType even isDivisibleBy: isInf isInfinite isNaN isNumber isZero negative odd positive sign strictlyPositive)
('converting' @ adaptToCollection:andSend: adaptToFloat:andSend: adaptToFraction:andSend: adaptToInteger:andSend: adaptToPoint:andSend: adaptToString:andSend: asInteger asNumber asPoint asSmallAngleDegrees asSmallPositiveDegrees degreesToRadians radiansToDegrees)
('intervals' to: to:by: to:by:do: to:do:)
('printing' defaultLabelForInspector isOrAreStringWith: newTileMorphRepresentative printOn: printStringBase: storeOn: storeOn:base: storeStringBase: stringForReadout)
('comparing' closeTo:)
('filter streaming' byteEncode:)
('as yet unclassified' reduce)"



! !

!NumberType methodsFor: 'initial value' stamp: 'sw 9/27/2001 17:29'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ (1 to: 9) atRandom! !

DataType subclass: #SoundType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!SoundType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:24'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #Sound! !

!SoundType methodsFor: 'initial value' stamp: 'sw 9/27/2001 17:29'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ 'croak'! !

DataType subclass: #StringType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!StringType methodsFor: 'initialization' stamp: 'ul 8/14/2015 18:04'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	
	super initialize.
	self vocabularyName: #String.

#((accessing 			'The basic info'
		(at: at:put: size endsWithDigit findString: findTokens: includesSubstring: indexOf: indexOf:startingAt: indexOf:startingAt:ifAbsent: lineCorrespondingToIndex: lineCount lineNumber: startsWithDigit numArgs))
(#'more accessing' 		'More basic info'
		(allButFirst allButFirst: allButLast allButLast: at:ifAbsent: atAllPut: atPin: atRandom: atWrap: atWrap:put: fifth first first: fourth from:to:put: last last: lastIndexOf: lastIndexOf:ifAbsent: middle replaceAll:with: replaceFrom:to:with: replaceFrom:to:with:startingAt: second sixth third))
(comparing				'Determining which comes first alphabeticly'
		(< <= = > >= beginsWith: endsWith: endsWithAnyOf: howManyMatch: match:))
(testing 				'Testing'
		(includes: isEmpty ifNil: ifNotNil: isAllDigits isAllSeparators isString lastSpacePosition))
(converting 			'Converting it to another form'
		(asCharacter asDate asInteger asLowercase asNumber asString asStringOrText asSymbol asText asTime asUppercase asUrl capitalized keywords numericSuffix romanNumber reversed splitInteger surroundedBySingleQuotes withBlanksTrimmed withSeparatorsCompacted withoutTrailingBlanks withoutTrailingDigits asSortedCollection))
(copying 				'Make another one like me'
		(copy copyFrom:to: copyUpTo: copyUpToLast: shuffled))
(enumerating		'Passing over the letters'
		(collect: collectWithIndex: do: from:to:do: reverseDo: select: withIndexDo: detect: detect:ifNone:))
) do: [:item | | aMethodCategory | 
			aMethodCategory := ElementCategory new categoryName: item first.
			aMethodCategory documentation: item second.
			item third do:
				[:aSelector | | aMethodInterface | 
					aMethodInterface := MethodInterface new initializeFor: aSelector.
					self atKey: aSelector putMethodInterface: aMethodInterface.
					aMethodCategory elementAt: aSelector put: aMethodInterface].
			self addCategory: aMethodCategory].
! !

!StringType methodsFor: 'initial value' stamp: 'sw 9/27/2001 17:29'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ 'abc'! !

DataType subclass: #SymbolListType
	instanceVariableNames: 'symbols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!SymbolListType commentStamp: 'sw 1/6/2005 17:52' prior: 0!
A type whose values range across a finite set of symbols, which are held in the "symbols" instance variable.!

SymbolListType subclass: #ButtonPhaseType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!ButtonPhaseType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:23'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	self vocabularyName: #ButtonPhase.
	symbols := #(buttonDown whilePressed buttonUp)! !

!ButtonPhaseType methodsFor: 'queries' stamp: 'mir 7/15/2004 10:35'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^true! !

!SymbolListType methodsFor: 'tiles' stamp: 'sw 12/3/2001 19:14'!
choices
	"answer the list of choices to offer as variant values"

	^ symbols copy! !

!SymbolListType methodsFor: 'initial value' stamp: 'sw 12/3/2001 19:27'!
initialValueForASlotFor: aPlayer
	"Answer the value to give initially to a newly created slot of the given type in the given player"

	^ self choices first! !

!SymbolListType methodsFor: 'tiles' stamp: 'sw 1/6/2005 17:24'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^ #(BorderStyle ButtonPhase TrailStyle) includes: vocabularyName! !

!SymbolListType methodsFor: 'tiles' stamp: 'sw 11/16/2001 07:30'!
symbols: symbolList
	"Set the receiver's list of symbols as indicated"

	symbols := symbolList! !

DataType subclass: #UnknownType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Protocols-Type Vocabularies'!

!UnknownType methodsFor: 'initialization' stamp: 'sw 9/27/2001 17:25'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	vocabularyName := #unknown! !

!UnknownType methodsFor: 'queries' stamp: 'sw 9/27/2001 13:37'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^ false! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'mir 7/15/2004 10:55'!
addStandardVocabulary: aVocabulary
	"Add a vocabulary to the list of standard vocabularies"

	self allStandardVocabularies at: aVocabulary vocabularyName put: aVocabulary! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'mir 7/15/2004 10:27'!
allStandardVocabularies
	"Answer a list of the currently-defined vocabularies in my AllStandardVocabularies list"
	"Vocabulary allStandardVocabularies"

	^AllStandardVocabularies ifNil: [AllStandardVocabularies := IdentityDictionary new].

! !

!Vocabulary class methodsFor: 'class initialization' stamp: 'nice 12/27/2009 03:11'!
embraceAddedTypeVocabularies
	"If there are any type-vocabulary subclases not otherwise accounted for, acknowledge them at this time"

	
	DataType allSubclasses do:
		[:dataType | | vocabulary |
			vocabulary := dataType new.
			vocabulary representsAType
				ifTrue: [(self allStandardVocabularies includesKey: vocabulary vocabularyName)
					ifFalse: 	[self addStandardVocabulary: vocabulary]]]! !

!Vocabulary class methodsFor: 'universal vocabularies' stamp: 'mir 7/15/2004 10:52'!
fullVocabulary
	"Answer the full vocabulary in my AllStandardVocabularies list, creating it if necessary"

	^ self allStandardVocabularies at: #Full ifAbsentPut: [FullVocabulary new]! !

!Vocabulary class methodsFor: 'eToy vocabularies' stamp: 'sw 9/29/2007 02:26'!
gettersForbiddenFromWatchers
	"Answer getters that should not have watchers launched to them"

	^ #(colorSees copy isOverColor: seesColor: newClone getNewClone color:sees: touchesA: overlaps: overlapsAny: distanceToPlayer: bearingTo: bearingFrom:)! !

!Vocabulary class methodsFor: 'class initialization' stamp: 'mir 7/15/2004 19:10'!
initialize
	"Initialize a few standard vocabularies and place them in the AllVocabularies list.  Call this to update all vocabularies."

	self initializeStandardVocabularies.
	self embraceAddedTypeVocabularies.

	"Vocabulary initialize"

! !

!Vocabulary class methodsFor: 'class initialization' stamp: 'sw 3/11/2003 12:36'!
initializeSilently
	"Initialize a few standard vocabularies and place them in the AllVocabularies list."

	self initializeStandardVocabularies.
	self embraceAddedTypeVocabularies.

	"Vocabulary initializeSilently"

! !

!Vocabulary class methodsFor: 'class initialization' stamp: 'tfel 8/1/2016 15:08'!
initializeStandardVocabularies
	"Initialize a few standard vocabularies and place them in the AllStandardVocabularies list."

	AllStandardVocabularies := nil.
	Smalltalk at: #EToyVocabulary
		ifPresent:[:aClass| self addStandardVocabulary: aClass new].
	Smalltalk at: #EToyVectorVocabulary
		ifPresent:[:aClass| self addStandardVocabulary: aClass new].

	self addStandardVocabulary: self newPublicVocabulary.
	self addStandardVocabulary: FullVocabulary new.

	self addStandardVocabulary: self newQuadVocabulary.

	self addStandardVocabulary: ColorType new.
	self addStandardVocabulary: BooleanType new.
	self addStandardVocabulary: GraphicType new.
	Smalltalk at: #PlayerType
		ifPresent:[:aClass| self addStandardVocabulary: aClass new].
	self addStandardVocabulary: SoundType new.
	self addStandardVocabulary: StringType new.
	self addStandardVocabulary: MenuType new.
	self addStandardVocabulary: UnknownType new.
	Smalltalk at: #ScriptNameType
		ifPresent:[:aClass| self addStandardVocabulary: aClass new].
	Smalltalk at: #PointType
		ifPresent:[:aClass| self addStandardVocabulary: aClass new].

	self addStandardVocabulary: (SymbolListType new symbols: #(simple raised inset complexFramed complexRaised complexInset complexAltFramed complexAltRaised complexAltInset); vocabularyName: #BorderStyle; yourself).
	self addStandardVocabulary: (SymbolListType new symbols: #(lines arrows arrowheads dots); vocabularyName: #TrailStyle; yourself).
	self addStandardVocabulary: (SymbolListType new symbols: #(leftToRight rightToLeft topToBottom bottomToTop); vocabularyName: #ListDirection; yourself).

	self addStandardVocabulary: (SymbolListType new symbols: #(topLeft bottomRight center justified); vocabularyName: #ListCentering; yourself).

	self addStandardVocabulary: (SymbolListType new symbols: #(#center #topLeft #topRight #bottomLeft #bottomRight #topCenter #leftCenter #rightCenter #bottomCenter ); vocabularyName: #CellPositioning; yourself).

	self addStandardVocabulary: (SymbolListType new symbols: #(#none #localRect #localSquare #globalRect #globalSquare ); vocabularyName: #CellSpacing; yourself).

	self addStandardVocabulary: (SymbolListType new symbols: #(buttonDown whilePressed buttonUp); vocabularyName: #ButtonPhase; yourself).

	self addStandardVocabulary: (SymbolListType new symbols: #(rotate #'do not rotate' #'flip left right' #'flip up down'); vocabularyName: #RotationStyle; yourself).

	self addStandardVocabulary: (SymbolListType new symbols: #(rigid spaceFill shrinkWrap); vocabularyName: #Resizing; yourself).

	self addStandardVocabulary: self newSystemVocabulary.  "A custom vocabulary for Smalltalk -- still under development)"

	self numberVocabulary.  		"creates and adds it"
	"self wonderlandVocabulary."  	"creates and adds it"
	self vocabularyForClass: Time.   "creates and adds it"

	Smalltalk at: #KedamaPatchType ifPresent:[:aClass| 
		self addStandardVocabulary: (aClass new vocabularyName: #Patch; yourself).
	].
	self addStandardVocabulary: (SymbolListType new symbols: #(wrap stick bouncing); vocabularyName: #EdgeMode; yourself).
	self addStandardVocabulary: (SymbolListType new symbols: #(logScale linear color); vocabularyName: #PatchDisplayMode; yourself).
	
	self addStandardVocabulary: (SymbolListType new symbols: #(#top #'top right' #right #'bottom right' #bottom #'bottom left' #left #'top left' #center ); vocabularyName: #AttachmentEdge; yourself).
	
	Smalltalk
		at: #CalendarMorph
		ifPresent: [:aClass | aClass assureDateFormatEstablished].

	"Vocabulary initialize"! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'sw 9/25/2001 20:13'!
newNumberVocabulary
	"Answer a Vocabulary object representing the Number vocabulary to the list of AllVocabularies"

	^ NumberType new! !

!Vocabulary class methodsFor: 'universal vocabularies' stamp: 'sw 12/14/2000 17:29'!
newPublicVocabulary
	| aVocabulary |
	"Answer a public vocabulary"

	aVocabulary := ScreenedVocabulary new.
	aVocabulary vocabularyName: #Public.
	aVocabulary documentation: '"Public" is vocabulary that excludes categories that start with "private" and methods that start with "private" or "pvt"'.

	aVocabulary categoryScreeningBlock: [:aCategoryName | (aCategoryName beginsWith: 'private') not].
	aVocabulary methodScreeningBlock: [:aSelector | 
		((aSelector beginsWith: 'private') or: [aSelector beginsWith: 'pvt']) not].
	^ aVocabulary
! !

!Vocabulary class methodsFor: 'testing and demo' stamp: 'sw 9/26/2001 03:51'!
newQuadVocabulary
	"Answer a Quad vocabulary -- something to mess with, to illustrate and explore ideas.  Applies to Quadrangles only."

	| aVocabulary  |
	aVocabulary := Vocabulary new vocabularyName: #Quad.
	aVocabulary documentation: 'A highly restricted test vocabulary that can be used with Quadrangle objects'.
	aVocabulary initializeFromTable:  #(
(borderColor borderColor: () Color (basic color) 'The color of the border' unused updating)
(borderWidth borderWidth: () Number (basic geometry) 'The width of the border' unused updating)
(insideColor insideColor: () Color (basic color) 'The color of the quadrangle' unused updating)
(display none () none (basic display) 'Display the quadrangle directly on the screen')
(width none () Number (geometry) 'The width of the object' unused updating)
(left setLeft: () Number (geometry) 'The left edge' unused updating)
(right setRight: () Number (geometry) 'The right edge' unused updating)
(width setWidth: () Number (geometry) 'The width of the object' unused updating)
(height setHeight: () Number (geometry) 'The height of the object' unused updating)
(hasPositiveExtent none () Boolean (tests) 'Whether the corner is to the lower-right of the origin' unused updating)
(isTall none () Boolean (tests) 'Whether the height is greater than the width' unused updating)).

	^ aVocabulary

"Vocabulary initialize"
"Quadrangle exampleInViewer"! !

!Vocabulary class methodsFor: 'eToy vocabularies' stamp: 'fbs 7/25/2013 07:21'!
newSystemVocabulary
	"Answer a Vocabulary object representing significant requests one can make to the Smalltalk object"

	| aVocabulary |
	aVocabulary := self new.

	aVocabulary vocabularyName: #System.
	aVocabulary documentation: 'Useful messages you might want to send to the current Smalltalk image'.
	aVocabulary initializeFromTable:  #(
(aboutThisSystem none () none (basic queries) 'put up a message describing the system' unused)
(saveAsNewVersion none () none (services) 'advance to the next available image-version number and save the image under that new name' unused znak)
(datedVersion none () String (queries) 'the version of the Squeak system')
(endianness none () String (queries) 'big or little - the byte-ordering of the hardware Squeak is currently running on')
(exitToDebugger none () none (dangerous) 'exits to the host debugger.  Do not use this -- I guarantee you will be sorry.')
(bytesLeft none () Number (basic services) 'perform a garbage collection and answer the number of bytes of free space remaining in the system')
"(browseAllCallsOn: none ((aSelector String)) none (#'queries') 'browse all calls on a selector')
(browseAllImplementorsOf: none ((aSelector String)) none (#'queries') 'browse all implementors of a selector')"

"(allMethodsWithSourceString:matchCase: none ((aString String) (caseSensitive Boolean)) none (queries) 'browse all methods that have the given source string, making the search case-sensitive or not depending on the argument provided.')

(browseMethodsWithString:matchCase: none ((aString String) (caseSensitive Boolean)) none (queries) 'browse all methods that contain the given string in any string literal, making the search case-sensitive or not depending on the argument provided.')

(browseAllImplementorsOf:localTo: none ((aSelector String) (aClass Class)) none (#'queries') 'browse all implementors of a selector that are local to a class')"

).
"(isKindOf: none 	((aClass Class)) Boolean (#'class membership') 'answer whether the receiver''s superclass chain includes aClass')"
	^ aVocabulary

"Vocabulary initialize"
"Vocabulary addStandardVocabulary: Vocabulary newSystemVocabulary"

"Smalltalk basicInspect"
"Smalltalk beViewed"
! !

!Vocabulary class methodsFor: 'testing and demo' stamp: 'sw 9/26/2001 11:59'!
newTestVocabulary
	"Answer a Test vocabulary -- something to mess with, to illustrate and explore ideas."

	| aVocabulary  |
	aVocabulary := Vocabulary new vocabularyName: #Test.
	aVocabulary documentation: 'An illustrative vocabulary for testing'.
	aVocabulary initializeFromTable:  #(
(isKindOf: none 	((aClass Class)) Boolean (#'class membership') 'answer whether the receiver''s superclass chain includes aClass')
(class none none Class (#'class membership' wimpy) 'answer the the class to which the receiver belongs')
(respondsTo: none ((aSelector Symbol))	Boolean (#'class membership') 'answer whether the receiver responds to the given selector')
(as:	none ((aClass Class)) Object (conversion) 'answer the receiver converted to be a member of aClass')).

	^ aVocabulary
"
	#((#'class membership' 	'Whether an object can respond to a given message, etc.' 	(isKindOf: class respondsTo:))
	(conversion 			'Messages to convert from one kind of object to another' 		(as:  asString))
	(copying				'Messages for making copies of objects'						(copy copyFrom:))
	(equality 				'Testing whether two objects are equal' 						( = ~= == ~~))
	(dependents				'Support for dependency notification'						(addDependent: removeDependent: release))) do:

		[:item | 
			aMethodCategory := ElementCategory new categoryName: item first.
			aMethodCategory documentation: item second.
			item third do:
				[:aSelector | 
					aMethodInterface := MethodInterface new initializeFor: aSelector.
					aVocabulary atKey: aSelector putMethodInterface: aMethodInterface.
					aMethodCategory elementAt: aSelector put: aMethodInterface].
			aVocabulary addCategory: aMethodCategory]."
! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'nice 12/27/2009 03:11'!
newTimeVocabulary
	"Answer a Vocabulary object representing me" 
	| aVocabulary |
	"Vocabulary newTimeVocabulary"
	"Vocabulary addStandardVocabulary: Vocabulary newTimeVocabulary"

	aVocabulary := self new vocabularyName: #Time.
	aVocabulary documentation: 'Time knows about hours, minutes, and seconds.  For long time periods, use Date'.

#((accessing 			'The basic info'
		(hours minutes seconds))
(arithmetic 				'Basic numeric operations'
		(addTime: subtractTime: max: min: min:max:))
(comparing				'Determining which is larger'
		(= < > <= >= ~= between:and:))
(testing 				'Testing'
		(ifNil: ifNotNil:))
(printing 				'Return a string for this Time'
		(hhmm24 print24 intervalString printMinutes printOn:))
(converting 			'Converting it to another form'
		(asSeconds asString))
(copying 				'Make another one like me'
		(copy))
) do: [:item | | aMethodCategory | 
			aMethodCategory := ElementCategory new categoryName: item first.
			aMethodCategory documentation: item second.
			item third do:
				[:aSelector | | aMethodInterface | 
					aMethodInterface := MethodInterface new initializeFor: aSelector.
					aVocabulary atKey: aSelector putMethodInterface: aMethodInterface.
					aMethodCategory elementAt: aSelector put: aMethodInterface].
			aVocabulary addCategory: aMethodCategory].
	#(#addTime: subtractTime: max: min: = < > <= >= ~= ) do: [:sel |
		(aVocabulary methodInterfaceAt: sel ifAbsent: [self error: 'fix this method']) 
			argumentVariables: (OrderedCollection with:
				(Variable new name: nil type: aVocabulary vocabularyName))].
	^ aVocabulary! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'mir 7/15/2004 10:31'!
numberVocabulary
	"Answer the standard vocabulary representing numbers, creating it if necessary"

	^self allStandardVocabularies at: #Number ifAbsentPut: [self newNumberVocabulary]! !

!Vocabulary class methodsFor: 'testing and demo' stamp: 'mir 7/15/2004 10:53'!
quadVocabulary
	"Answer the Quad vocabulary lurking in my AllStandardVocabularies list, creating it if necessary"
	"Vocabulary quadVocabulary"

	^ self allStandardVocabularies at: #Quad ifAbsentPut: [self newQuadVocabulary]! !

!Vocabulary class methodsFor: 'testing and demo' stamp: 'mir 7/15/2004 10:54'!
testVocabulary
	"Answer the Test vocabulary lurking in my AllStandardVocabularies list, creating it if necessary"
	"Vocabulary testVocabulary"

	^ self allStandardVocabularies at: #Test ifAbsentPut: [self newTestVocabulary]! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'cmm 9/9/2015 17:42'!
typeChoices
	"Answer a list of all user-choosable data types"

	| vocabulariesForType |
	vocabulariesForType := self allStandardVocabularies select: [:e | e representsAType].
	(Preferences valueOfFlag: #allowEtoyUserCustomEvents) ifFalse: [vocabulariesForType removeKey: #CustomEvents ifAbsent: []].
	^vocabulariesForType keys sort! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'mir 7/15/2004 10:44'!
vocabularyForClass: aClass
	"Answer the standard vocabulary for that class.  Create it if not present and init message exists.  Answer nil if none exists and no init message present."

	| initMsgName newTypeVocab |
	(self allStandardVocabularies includesKey: aClass name)
		ifTrue: [^self allStandardVocabularies at: aClass name].

	initMsgName := ('new', aClass name, 'Vocabulary') asSymbol.
	^(self respondsTo: initMsgName)
		 ifTrue:	[
			newTypeVocab := self perform: initMsgName.
			self addStandardVocabulary: newTypeVocab.
			newTypeVocab]
		ifFalse: [nil]! !

!Vocabulary class methodsFor: 'type vocabularies' stamp: 'mir 7/15/2004 10:59'!
vocabularyForType: aType
	"Answer a vocabulary appropriate to the given type, which is normally going to be a symbol such as #Number or #Color.  Answer the Unknown vocabulary as a fall-back"

	| ucSym |
	(aType isKindOf: Vocabulary) ifTrue: [^ aType].
	ucSym := aType capitalized asSymbol.
	^self allStandardVocabularies at: ucSym ifAbsent: [self vocabularyNamed: #unknown]! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'sw 6/4/2001 16:33'!
vocabularyFrom: aNameOrVocabulary
	"Answer the standard vocabulary of the given name, or nil if none found,  For backward compatibilitythe parameter might be an actual vocabulary, in which case return it"

	(aNameOrVocabulary isKindOf: Vocabulary) ifTrue: [^ aNameOrVocabulary].
	^ self vocabularyNamed: aNameOrVocabulary! !

!Vocabulary class methodsFor: 'standard vocabulary access' stamp: 'nk 8/22/2004 09:24'!
vocabularyNamed: aName
	"Answer the standard vocabulary of the given name, or nil if none found"

	^ self allStandardVocabularies at: aName asSymbol ifAbsent: []! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 1/26/2001 23:03'!
addCategoryNamed: aCategoryName
	"Add a category of the given name to my categories list,"

	categories add: (ElementCategory new categoryName: aCategoryName asSymbol)! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 12/11/2000 15:31'!
addCategory: aCategory
	"Add the given category to my categories list"

	categories add: aCategory! !

!Vocabulary methodsFor: 'initialization' stamp: 'nice 12/27/2009 03:11'!
addFromTable: aTable
	"Add each method-specification tuples, each of the form:
		(1)	selector
		(2)	companion setter selector (#none or nil indicate none)
		(3)  argument specification array, each element being an array of the form
				<arg name>  <arg type>
		(4)  result type, (#none or nil indicate none)
		(5)  array of category symbols, i.e. the categories in which this element should appear.
		(6)  help message. (optional)
		(7)  wording (optional)
		(8)  auto update flag (optional) - if #updating, set readout to refetch automatically

	Make new categories as needed.
	Consult Vocabulary class.initializeTestVocabulary for an example of use"
				
	
	aTable do:
		[:tuple |   tuple fifth do: [:aCategorySymbol | | aSelector aMethodInterface aMethodCategory doc wording |
			(aMethodCategory := self categoryAt: aCategorySymbol) ifNil: [ 
					aMethodCategory := ElementCategory new categoryName: aCategorySymbol.
					self addCategory: aMethodCategory].		
		
			aMethodInterface := MethodInterface new.
			aSelector := tuple first.
			aMethodInterface selector: aSelector type: tuple fourth setter: tuple second.
			aMethodCategory elementAt: aSelector put: aMethodInterface.
			self atKey: aSelector putMethodInterface: aMethodInterface.
			((tuple third ~~ #none) and: [tuple third isEmptyOrNil not])
				ifTrue:
					[aMethodInterface argumentVariables: (tuple third collect:
						[:pair | Variable new name: pair first type: pair second])].

		
			doc := (tuple size >= 6 and: [(#(nil none unused) includes: tuple sixth) not])
				ifTrue:
					[tuple sixth]
				ifFalse:
					[nil].
			wording := (tuple size >= 7 and: [(#(nil none unused) includes: tuple seventh) not])
				ifTrue:
					[tuple seventh]
				ifFalse:
					[aSelector].
			aMethodInterface
				wording: wording;
				helpMessage: doc.
			tuple size >= 8 ifTrue:
				[aMethodInterface setToRefetch]]].
! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 1/30/2001 16:31'!
allCategoryName
	"Answer the name by which the 'all' category is known.  This is redundant with two other things, including ClassOrganizer allCategory, at the moment -- should be cleaned up someday."

	^ '-- all --' asSymbol! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 2/24/2001 01:02'!
allMethodsInCategory: categoryName 
	"Answer a list of methods in the category of the given name"

	^ self allMethodsInCategory: categoryName forInstance: object ofClass: object class! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 1/30/2001 16:31'!
allMethodsInCategory: categoryName forInstance: anObject ofClass: aClass
	"Answer a list of all methods in the etoy interface which are in the given category, on behalf of aClass and possibly anObject.  Note that there is no limitClass at play here."

	| aCategory |
	categoryName ifNil: [^ OrderedCollection new].
	categoryName = self allCategoryName ifTrue:
		[^ methodInterfaces collect: [:anInterface | anInterface selector]].

	aCategory := categories detect: [:cat | cat categoryName == categoryName asSymbol] ifNone: [^ OrderedCollection new].
	^ aCategory elementsInOrder collect: [:anElement | anElement selector] thenSelect:
			[:aSelector | aClass canUnderstand: aSelector]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 1/26/2001 16:22'!
allSelectorsInVocabulary
	"Answer a list of all selectors in the vocabulary"

	^ methodInterfaces collect: [:m | m selector]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 9/27/2001 17:47'!
asSymbol
	"Answer a symbol representing the receiver"

	^ self vocabularyName! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 9/13/2001 14:29'!
atKey: aKey putMethodInterface: anInterface
	"Place the given interface at the given key."

	methodInterfaces at: aKey put: anInterface! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 12/13/2000 20:02'!
categories
	"Answer a list of the categories in the receiver"

	^ categories! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 12/12/2000 06:13'!
categoriesContaining: aSelector forClass: aClass
	"Answer a list of categories that include aSelector"

	^ self categories select:
		[:aCategory | aCategory includesKey: aSelector]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 12/11/2000 15:31'!
categoryAt: aSymbol
	"Answer the category which has the given symbol as its categoryName, else nil if none found"

	^ categories detect: [:aCategory | aCategory categoryName == aSymbol] ifNone: [nil]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 1/30/2001 16:31'!
categoryCommentFor: aCategoryName
	"Answer diocumentation for the given category name, a symbol"

	categories do:
		[:cat | cat categoryName == aCategoryName ifTrue: [^ cat documentation]].

	aCategoryName = self allCategoryName ifTrue:
		[^ 'Shows all methods, whatever other categories they may belong to'].
	#(
	(all					'Danger!! An old designation that usually does NOT include all of anything!!')
	('as yet unclassified'	'Methods not yet given a specific classification in some class in which they are implemented')
	(private				'Methods that should only be called by self'))

		do:
			[:pair | pair first = aCategoryName ifTrue: [^ pair second]].

	^ aCategoryName, ' is a category that currently has no documentation'
! !

!Vocabulary methodsFor: 'queries' stamp: 'nice 12/27/2009 03:11'!
categoryList
	"Answer the category list considering only code implemented in my 
	limitClass and lower. This variant is used when the limitClass and 
	targetObjct are known"
	| classToUse |
	classToUse := object class.
	^ categories
		select: [:aCategory | | foundAMethod | 
			foundAMethod := false.
			aCategory elementsInOrder
				do: [:aSpec | | classThatImplements | 
					classThatImplements := classToUse whichClassIncludesSelector: aSpec selector.
					(classThatImplements notNil
							and: [classThatImplements includesBehavior: limitClass])
						ifTrue: [foundAMethod := true]].
			foundAMethod]
		thenCollect: [:aCategory | aCategory categoryName]! !

!Vocabulary methodsFor: 'queries' stamp: 'nice 12/27/2009 03:11'!
categoryListForInstance: targetObject ofClass: aClass limitClass: mostGenericClass 
	"Answer the category list for the given instance (may be nil) of the 
	given class, considering only code implemented in mostGenericClass and 
	lower "
	| classToUse |
	classToUse := targetObject
				ifNil: [aClass]
				ifNotNil: [targetObject class].
	^ categories
		select: [:aCategory | | foundAMethod | 
			foundAMethod := false.
			aCategory elementsInOrder
				do: [:aSpec | | classThatImplements | 
					classThatImplements := classToUse whichClassIncludesSelector: aSpec selector.
					(classThatImplements notNil
							and: [classThatImplements includesBehavior: mostGenericClass])
						ifTrue: [foundAMethod := true]].
			foundAMethod]
		thenCollect: [:aCategory | aCategory categoryName]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 9/12/2001 22:43'!
categoryWhoseTranslatedWordingIs: aWording
	"Answer the category whose translated is the one provided, or nil if none"

	^ self categories detect: [:aCategory | aCategory wording = aWording] ifNone: [nil]
! !

!Vocabulary methodsFor: 'queries' stamp: 'nice 12/27/2009 04:10'!
categoryWithNameIn: categoryNames thatIncludesSelector: aSelector forInstance: targetInstance ofClass: targetClass
	"Answer the name of a category, from among the provided categoryNames, which defines the selector for the given class.  Note reimplementor"

	
	self categories do:
		[:aCategory | | itsName |
		((categoryNames includes: (itsName := aCategory categoryName)) and:  [aCategory includesKey: aSelector])
			ifTrue:
				[^ itsName]].
	^ nil! !

!Vocabulary methodsFor: 'queries' stamp: 'mir 7/15/2004 15:15'!
categoryWordingAt: aSymbol
	"Answer the wording for the category at the given symbol"

	| result |
	result := self categoryAt: aSymbol.
	^result
		ifNotNil: [result wording]
		ifNil: [aSymbol]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 12/14/2000 05:55'!
classToUseFromInstance: anInstance ofClass: aClass
	"A small convenience to assist in complications arising because an instance is sometimes provided and sometimes not"

	^ anInstance ifNotNil: [anInstance class] ifNil: [aClass]
! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 12/12/2000 06:06'!
encompassesAPriori: aClass
	"Answer whether the receiver  a priori encompasses aClass -- see implementors"

	^ false! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 3/20/2001 15:42'!
includesDefinitionForSelector: aSelector
	"Answer whether the given selector is known to the vocabulary.  This is independent of whether its definition lies within the range specified by my limitClass.  Answer whether the given selector is known to the vocabulary.  Unsent at the moment, may disappear."

	^ methodInterfaces includesKey: aSelector! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 12/1/2000 21:57'!
includesSelector: aSelector
	"Answer whether the given selector is known to the vocabulary"

	^ methodInterfaces includesKey: aSelector! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 3/19/2001 23:56'!
includesSelector: aSelector forInstance: anInstance ofClass: aTargetClass limitClass: mostGenericClass
	"Answer whether the vocabulary includes the given selector for the given class (and instance, if provided), only considering method implementations in mostGenericClass and lower"

	| classToUse aClass |

	(methodInterfaces includesKey: aSelector) ifFalse: [^ false].
	classToUse := self classToUseFromInstance: anInstance ofClass: aTargetClass.
	^ (aClass := classToUse whichClassIncludesSelector: aSelector)
		ifNil:
			[false]
		ifNotNil:
			[(aClass includesBehavior: mostGenericClass) and:
				[(self someCategoryThatIncludes: aSelector) notNil]]
! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 9/12/2001 15:16'!
initialize
	"Initialize the receiver (automatically called when instances are created via 'new')"

	super initialize.
	vocabularyName := #unnamed.
	categories := OrderedCollection new.
	methodInterfaces := IdentityDictionary new! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 2/21/2001 15:39'!
initializeFor: anObject
	"Initialize the receiver to bear a vocabulary suitable for anObject"

	object := anObject.
	vocabularyName := #unnamed.
	categories := OrderedCollection new.
	methodInterfaces := IdentityDictionary new.
	self documentation: 'A vocabulary that has not yet been documented'.
! !

!Vocabulary methodsFor: 'initialization' stamp: 'nice 12/27/2009 04:09'!
initializeFromTable: aTable
	"Initialize the receiver from a list of method-specification tuples, each of the form:
		(1)	selector
		(2)	companion setter selector (#none or nil indicate none)
		(3)  argument specification array, each element being an array of the form
				<arg name>  <arg type>
		(4)  result type, (#none or nil indicate none)
		(5)  array of category symbols, i.e. the categories in which this element should appear.
		(6)  help message. (optional)
		(7)  wording (optional)
		(8)  auto update flag (optional) - if #updating, set readout to refetch automatically

	Consult Vocabulary class.initializeTestVocabulary for an example of use"
				
	| categoryList |
	categoryList := Set new.
	aTable do:
		[:tuple | categoryList addAll: tuple fifth].
	categoryList := categoryList asSortedArray.
	categoryList do:
		[:aCategorySymbol | | aMethodCategory |
			aMethodCategory := ElementCategory new categoryName: aCategorySymbol.
			aTable do:
				[:tuple | | doc aSelector wording aMethodInterface |
				(tuple fifth includes: aCategorySymbol) ifTrue:
					[aMethodInterface := MethodInterface new.
					aSelector := tuple first.
					aMethodInterface selector: aSelector type: tuple fourth setter: tuple second.
					aMethodCategory elementAt: aSelector put: aMethodInterface.
					self atKey: aSelector putMethodInterface: aMethodInterface.
					((tuple third ~~ #none) and: [tuple third isEmptyOrNil not])
						ifTrue:
							[aMethodInterface argumentVariables: (tuple third collect:
								[:pair | Variable new name: pair first type: pair second])].
					doc := (tuple size >= 6 and: [(#(nil none unused) includes: tuple sixth) not])
						ifTrue:
							[tuple sixth]
						ifFalse:
							[nil].
 					wording := (tuple size >= 7 and: [(#(nil none unused) includes: tuple seventh) not])
						ifTrue:
							[tuple seventh]
						ifFalse:
							[aSelector asString].
					aMethodInterface
						wording: wording;
						helpMessage: doc.
					tuple size >= 8 ifTrue:
						[aMethodInterface setToRefetch]]].
			self addCategory: aMethodCategory]! !

!Vocabulary methodsFor: 'queries' stamp: 'nk 9/1/2004 08:42'!
isEToyVocabulary
	^false! !

!Vocabulary methodsFor: 'queries' stamp: 'ul 8/2/2011 20:13'!
methodInterfaceAt: aSelector ifAbsent: aBlock
	"Answer the vocabulary's method interface for the given selector; if absent, return the result of evaluating aBlock"

	^methodInterfaces at: aSelector ifAbsent: aBlock! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 8/11/2002 02:25'!
methodInterfaceForSelector: aSelector class: aClass
	"Answer a method interface for the selector"

	^ self methodInterfaceAt: aSelector ifAbsent:
		[MethodInterface new conjuredUpFor: aSelector class: aClass]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 5/25/2001 10:42'!
methodInterfacesDo: aBlock
	"Evaluate aBlock on behalf, in turn, of each of my methodInterfaces"

	methodInterfaces do: aBlock

	! !

!Vocabulary methodsFor: 'queries' stamp: 'nice 12/27/2009 03:11'!
methodInterfacesInCategory: categoryName forInstance: anObject ofClass: aClass limitClass: aLimitClass
	"Answer a list of method interfaces of all methods in the given category, provided they are implemented no further away than aLimitClass."

	
	^ ((self allMethodsInCategory: categoryName forInstance: anObject ofClass: aClass) collect:
		[:sel | methodInterfaces at: sel ifAbsent:
			[MethodInterface new conjuredUpFor: sel class: aClass]]) select:
				[:aMethodInterface | | defClass |
					defClass := aClass whichClassIncludesSelector: aMethodInterface selector.
					(defClass notNil and: [defClass includesBehavior: aLimitClass])]! !

!Vocabulary methodsFor: 'method list' stamp: 'sw 11/13/2001 09:34'!
phraseSymbolsToSuppress
	"Answer a dictatorially-imposed list of phrase-symbols that are to be suppressed from viewers, even if they otherwise show up.  Note that EToyVocabulary reimplements"

	^ #()! !

!Vocabulary methodsFor: 'printing' stamp: 'sw 12/1/2000 22:05'!
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that   identifies the receiver."

	super printOn: aStream.
	vocabularyName ifNotNil: [aStream nextPutAll: ' named "', vocabularyName, '"']! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 5/25/2001 10:42'!
renameCategoryFrom: oldName to: newName
	"Rename the category currently known by oldName to be newName.  No senders at present but once a UI is establshed for renaming categories, this will be useful."

	| aCategory |
	(aCategory := self categoryAt: oldName) ifNil: [^ self].
	aCategory categoryName: newName! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 9/27/2001 03:24'!
representsAType
	"Answer whether this vocabulary represents an end-user-sensible data type"

	^ false! !

!Vocabulary methodsFor: 'private' stamp: 'nice 12/27/2009 03:11'!
setCategoryStrings: categoryTriplets
	"Establish the category strings as per (internalCategorySymbol newCategoryWording balloon-help)"

	
	categoryTriplets do:
		[:triplet | | category |
			(category := self categoryAt: triplet first) ifNotNil: [
				category wording: triplet second.
				category helpMessage: triplet third]]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 3/19/2001 23:55'!
someCategoryThatIncludes: aSelector
	"Answer the name of a category that includes the selector, nil if none"

	^ categories detect: [:c | c includesKey: aSelector] ifNone: [nil]! !

!Vocabulary methodsFor: 'initialization' stamp: 'nice 10/21/2009 00:17'!
strings
	| strm |
	"Get started making a vocabulary for a foreign language.  That is, build a method like #addGermanVocabulary, but for another language.  
	Returns this vocabulary in the same form used as the input used for foreign languages.  To avoid string quote problems, execute
	Transcript show: Vocabulary eToyVocabulary strings.
and copy the text from the transcript to the method you are building."

	"selector		wording			documentation"

strm := WriteStream on: (String new: 400).
methodInterfaces keys asArray sort do: [:sel |
	strm cr; nextPut: $(;
		nextPutAll: sel; tab; tab; tab; nextPut: $';
		nextPutAll: (methodInterfaces at: sel) wording;
		nextPut: $'; tab; tab; tab; nextPut: $';
		nextPutAll: (methodInterfaces at: sel) documentation;
		nextPut: $'; nextPut: $)].
^ strm contents! !

!Vocabulary methodsFor: 'translation' stamp: 'nice 9/11/2013 22:47'!
translatedWordingFor: aSymbol
	"If I have a translated wording for aSymbol, return it, else return aSymbol.  Caveat: at present, this mechanism is only germane for *assignment-operator wordings*"

	#(#':' #Incr: #Decr: #Mult:) with: #('' 'increase by' 'decrease by' 'multiply by') do:
		[:a :b | aSymbol = a ifTrue: [^ b translated]].

	^ aSymbol translated! !

!Vocabulary methodsFor: 'translation' stamp: 'sw 5/25/2001 10:45'!
translatedWordingsFor: symbolList
	"Answer a list giving the translated wordings for the input list. Caveat: at present, this mechanism is only germane for *categories*"

	^ symbolList collect: [:sym | self translatedWordingFor: sym]
! !

!Vocabulary methodsFor: 'translation' stamp: 'yo 7/2/2004 21:51'!
translationKeyFor: translatedWording

	self flag: #yo.
	^ translatedWording.
! !

!Vocabulary methodsFor: 'translation' stamp: 'mir 7/13/2004 00:42'!
translationTable
	^translationTable ifNil: [ElementCategory new]! !

!Vocabulary methodsFor: 'queries' stamp: 'sw 12/1/2000 22:05'!
vocabularyName
	"Answer the name of the vocabulary"

	^ vocabularyName! !

!Vocabulary methodsFor: 'initialization' stamp: 'sw 12/1/2000 22:06'!
vocabularyName: aName
	"Set the name of the vocabulary as indicated"

	vocabularyName := aName! !

!Object methodsFor: '*Protocols' stamp: 'dtl 2/27/2010 10:20'!
currentVocabulary
	"Answer the currently-prevailing default vocabulary."

	^ Project current currentVocabulary
! !

!Object methodsFor: '*Protocols' stamp: 'sw 1/30/2001 22:24'!
haveFullProtocolBrowsed
	"Open up a Lexicon on the receiver"

	^ self haveFullProtocolBrowsedShowingSelector: nil

	"(2@3) haveFullProtocolBrowsed"
! !

!Object methodsFor: '*Protocols' stamp: 'ar 9/27/2005 21:04'!
haveFullProtocolBrowsedShowingSelector: aSelector
	"Open up a Lexicon on the receiver, having it open up showing aSelector, which may be nil"

	| aBrowser |
	aBrowser := (Smalltalk at: #InstanceBrowser ifAbsent:[^nil]) new useVocabulary: Vocabulary fullVocabulary.
	aBrowser openOnObject: self inWorld: ActiveWorld showingSelector: aSelector

	"(2@3) haveFullProtocolBrowsed"! !
Vocabulary initialize!
